# coding: utf8


from typing import Optional, List

from ._enums import (
    AdjustInterestToPaymentDate,
    BusinessDayConvention,
    DateRollingConvention,
    DayCountBasis,
    Direction,
    Frequency,
    IndexAverageMethod,
    IndexCompoundingMethod,
    IndexObservationMethod,
    IndexResetType,
    IndexSpreadCompoundingMethod,
    InterestCalculationConvention,
    InterestType,
    NotionalExchange,
    StubRule,
)
from ._models import AmortizationItem
from ..._object_definition import ObjectDefinition
from ....._tools import try_copy_to_list


class LegDefinition(ObjectDefinition):
    """
    API endpoint for Financial Contract analytics,
    that returns calculations relevant to each contract type.

    Parameters
    ----------
    instrument_tag : str, optional

    leg_tag : str, optional
        A user-defined string to identify the direction of the leg: 'paid' or
        'received'. optional. no default value applies.
    direction : Direction, optional
        The indication whether the cash flows of the instrument's leg are paid or
        received. the possible values are:   paid: the cash flows are paid to the
        counterparty,   received: the cash flows are received from the counterparty.  no
        default value applies.
    interest_type : InterestType, optional
        An indicator whether the instrument pays a fixed or floating interest. the
        possible values are: fixed, float. no default value applies.
    notional_ccy : str, optional
        The currency of the instrument's notional amount. the value is expressed in iso
        4217 alphabetical format (e.g. 'usd'). no default value applies.
    notional_amount : float, optional
        The notional amount of the instrument's leg. the default value is '1,000,000'.
    fixed_rate_percent : float, optional
        The interest rate of the instrument. the value is expressed in percentages.
        mandatory if no instrumentcode is defined. if instrumentcode is defined, the
        value comes from the instrument reference data.
    index_name : str, optional
        The name of the floating rate index (e.g. 'euribor'). no default value applies.
    index_tenor : str, optional
        The period code indicating the maturity of the floating rate index. the default
        value is the tenor equivalent toindexresetfrequency or interestpaymentfrequency.
    spread_bp : float, optional
        The interest spread in basis points that is added to the floating rate index
        value. optional. if instrumentcode is defined, the value comes from the
        instrument reference data. in case of a user-defined instrument, the default
        value is '0'.
    interest_payment_frequency : Frequency, optional
        The frequency of the interest payment. either indexresetfrequency or
        interestpaymentfrequency must be provided (e.g. annual, semiannual).   the
        default value is indexresetfrequency.
    interest_calculation_method : DayCountBasis, optional
        The day count basis method used to calculate the interest payments(e.g.
        dcb_30_360, dcb_30_actual). the default value is selected based onnotionalccy.
    accrued_calculation_method : DayCountBasis, optional
        The day count basis method used to calculate the accrued interest payments (e.g.
        dcb_30_360, dcb_30_actual).   if instrumentcode is defined, the value comes from
        the instrument reference data. in case of a user-defined instrument,
        interestcalculationmethod is used.
    payment_business_day_convention : BusinessDayConvention, optional
        The method to adjust dates to working days. the possible values are:
        previousbusinessday,    nextbusinessday,    modified following,    nomoving,
        bbswmodifiedfollowing.   if instrumentcode is defined, the value comes from the
        instrument reference data. in case of a user-defined instrument, the default
        value is'modifiedfollowing'.
    payment_roll_convention : DateRollingConvention, optional
        The method to adjust payment dates when they fall at the end of the month (e.g.
        28th of february, 30th, 31st). the possible values are:    last,    same,
        last28,    same28.   if instrumentcode is defined, the value comes from the
        instrument reference data. in case of a user-defined instrument, the default
        value is'same'.
    index_reset_frequency : Frequency, optional
        The reset frequency for the floating instrument (e.g. annual, semiannual).   the
        default value is interestpaymentfrequency.
    index_reset_type : IndexResetType, optional
        A type indicating if the floating rate index is reset before the coupon period
        starts or at the end of the coupon period. the possible values are: inadvance:
        resets the index before the start of the interest period, inarrears: resets the
        index at the end of the interest period. the default value is 'inadvance'.
    index_fixing_lag : int, optional
        The number of working daysbetween the fixing date and the start of the coupon
        period ('inadvance') or the end of the coupon period ('inarrears'). the
        inadvance/inarrears mode is set in the indexresettype parameter. the default
        value is the fixing lag associated to the index defined/determined by default on
        the floating instrument.
    first_regular_payment_date : str, optional
        The first regular interest payment date used for the odd first interest period.
        the value is expressed in iso 8601 format: yyyy-mm-ddt[hh]:[mm]:[ss]z (e.g.
        2021-01-01t00:00:00z). no default value applies.
    last_regular_payment_date : str, optional
        The last regular interest payment date used for the odd last interest period.
        the value is expressed in iso 8601 format: yyyy-mm-ddt[hh]:[mm]:[ss]z (e.g.
        2021-01-01t00:00:00z). no default value applies.
    amortization_schedule : AmortizationItem, optional
        The amortization schedule of the instrument. it contains the following
        information:   startdate,   enddate,   remainingnotional,
        amortizationfrequency,   amount,   amortizationtype.  optional. no default value
        applies.
    payment_business_days : str, optional
        A list of comma-separated calendar codes to adjust dates (e.g. 'emu' or 'usa').
        the default value is the calendar associated to the market conventions of the
        interestpaymentccy for the corresponding leg.
    notional_exchange : NotionalExchange, optional
        An indicator if the notional amount is exchanged and when it is exchanged. the
        possible values are:    none,    start,    end,    both,    endadjustment.   the
        default value is 'none'.
    adjust_interest_to_payment_date : AdjustInterestToPaymentDate, optional
        An indication if the coupon dates are adjusted to the payment dates. the
        possible values are:   adjusted,   unadjusted.  if instrumentcode is defined,
        the value comes from the instrument reference data. in case of a user-defined
        instrument, the default value is 'adjusted'.
    index_compounding_method : IndexCompoundingMethod, optional
        The method how the interest rate is calculated from the reset floating rates
        when the reset frequency is higher than the interest payment frequency (e.g.
        daily index reset with quarterly interest payments). the possible values are:
        compounded, average, constant, adjustedcompounded, mexicancompounded. if
        instrumentcode is defined, the value comes from the instrument reference data.
        in case of a user-defined instrument, the default value is 'constant'.
    interest_payment_delay : int, optional
        The number of working days between the end of the interest accrual period and
        the interest payment date. by default, no delay (0) is applied.
    stub_rule : StubRule, optional
        The rule that defines whether coupon roll dates are aligned to the maturity or
        issue date. the possible values are:   issue,   maturity,   shortfirstprorata,
        shortfirstfull,   longfirstfull,   shortlastprorata.  the default value is
        'maturity'.
    index_average_method : IndexAverageMethod, optional
        The value of the average index calculation method. the possible values are:
        compoundedactual,      dailycompoundedaverage,      compoundedaveragerate,
        arithmeticaverage
    index_observation_method : IndexObservationMethod, optional
        (rfr) method for determining the accrual observation period. the possible values
        are:      lookback: use the interest period for both rate accrual and interest
        payment.      periodshift: use the observation period for both rate accrual and
        interest payment.      mixed: use the observation period for rate accrual and
        the interest period for interest payment.
    index_spread_compounding_method : IndexSpreadCompoundingMethod, optional
        The method defining how the computed float leg spread is applied to compounded
        rate. it applies only when indexcompoundingmethod= compounded. the possible
        values are:    isdacompounding,    nocompounding,    isdaflatcompounding.  the
        default value is 'isdacompounding'.
    interest_calculation_convention : InterestCalculationConvention, optional
        The day count basis method convention used to calculate the interest payments.
        optional. defaults to moneymarket. if instrumentcode is defined, the value comes
        from the instrument reference data.
    cms_template : str, optional
        A reference to a common swap contract that represents the underlying swap in
        case of a constant maturity swap contract (cms). example: eur_ab6e. no default
        value applies.
    floor_strike_percent : float, optional
        The contractual strike rate of the floor. the value is expressed in percentages.
        if this parameter is set, the floor will apply to the leg with the same
        parameters set in the swaplegdefinition (e.g.maturity, frequency, index,
        discounting rule). no default value applies.
    index_fixing_ric : str, optional
        The ric that carries the fixing value if the instrument has a floating interest.
        optional. mandatory for floating rate instruments if no instrumentcode is
        defined. if instrumentcode is defined, the value comes from the instrument
        reference data. no default value applies.
    upfront_amount : float, optional
        The amount which represents the net present value of the swap. it is computed as
        [(100  dirtypricepercent / 100) x notionalamount]. the value is expressed in
        upfrontamountccy. by default, no payment (0) applies.
    """

    def __init__(
        self,
        instrument_tag: Optional[str] = None,
        leg_tag: Optional[str] = None,
        direction: Optional[Direction] = None,
        interest_type: Optional[InterestType] = None,
        notional_ccy: Optional[str] = None,
        notional_amount: Optional[float] = None,
        fixed_rate_percent: Optional[float] = None,
        index_name: Optional[str] = None,
        index_tenor: Optional[str] = None,
        spread_bp: Optional[float] = None,
        interest_payment_frequency: Optional[Frequency] = None,
        interest_calculation_method: Optional[DayCountBasis] = None,
        accrued_calculation_method: Optional[DayCountBasis] = None,
        payment_business_day_convention: Optional[BusinessDayConvention] = None,
        payment_roll_convention: Optional[DateRollingConvention] = None,
        index_reset_frequency: Optional[Frequency] = None,
        index_reset_type: Optional[IndexResetType] = None,
        index_fixing_lag: Optional[int] = None,
        first_regular_payment_date: Optional[str] = None,
        last_regular_payment_date: Optional[str] = None,
        amortization_schedule: Optional[List[AmortizationItem]] = None,
        payment_business_days: Optional[str] = None,
        notional_exchange: Optional[NotionalExchange] = None,
        adjust_interest_to_payment_date: Optional[AdjustInterestToPaymentDate] = None,
        index_compounding_method: Optional[IndexCompoundingMethod] = None,
        interest_payment_delay: Optional[int] = None,
        stub_rule: Optional[StubRule] = None,
        index_average_method: Optional[IndexAverageMethod] = None,
        index_observation_method: Optional[IndexObservationMethod] = None,
        index_spread_compounding_method: Optional[IndexSpreadCompoundingMethod] = None,
        interest_calculation_convention: Optional[InterestCalculationConvention] = None,
        cms_template: Optional[str] = None,
        floor_strike_percent: Optional[float] = None,
        index_fixing_ric: Optional[str] = None,
        upfront_amount: Optional[float] = None,
    ) -> None:
        super().__init__()
        self.instrument_tag = instrument_tag
        self.leg_tag = leg_tag
        self.direction = direction
        self.interest_type = interest_type
        self.notional_ccy = notional_ccy
        self.notional_amount = notional_amount
        self.fixed_rate_percent = fixed_rate_percent
        self.index_name = index_name
        self.index_tenor = index_tenor
        self.spread_bp = spread_bp
        self.interest_payment_frequency = interest_payment_frequency
        self.interest_calculation_method = interest_calculation_method
        self.accrued_calculation_method = accrued_calculation_method
        self.payment_business_day_convention = payment_business_day_convention
        self.payment_roll_convention = payment_roll_convention
        self.index_reset_frequency = index_reset_frequency
        self.index_reset_type = index_reset_type
        self.index_fixing_lag = index_fixing_lag
        self.first_regular_payment_date = first_regular_payment_date
        self.last_regular_payment_date = last_regular_payment_date
        self.amortization_schedule = try_copy_to_list(amortization_schedule)
        self.payment_business_days = payment_business_days
        self.notional_exchange = notional_exchange
        self.adjust_interest_to_payment_date = adjust_interest_to_payment_date
        self.index_compounding_method = index_compounding_method
        self.interest_payment_delay = interest_payment_delay
        self.stub_rule = stub_rule
        self.index_average_method = index_average_method
        self.index_observation_method = index_observation_method
        self.index_spread_compounding_method = index_spread_compounding_method
        self.interest_calculation_convention = interest_calculation_convention
        self.cms_template = cms_template
        self.floor_strike_percent = floor_strike_percent
        self.index_fixing_ric = index_fixing_ric
        self.upfront_amount = upfront_amount

    @property
    def accrued_calculation_method(self):
        """
        The day count basis method used to calculate the accrued interest payments (e.g.
        dcb_30_360, dcb_30_actual).   if instrumentcode is defined, the value comes from
        the instrument reference data. in case of a user-defined instrument,
        interestcalculationmethod is used.
        :return: enum DayCountBasis
        """
        return self._get_enum_parameter(DayCountBasis, "accruedCalculationMethod")

    @accrued_calculation_method.setter
    def accrued_calculation_method(self, value):
        self._set_enum_parameter(DayCountBasis, "accruedCalculationMethod", value)

    @property
    def adjust_interest_to_payment_date(self):
        """
        An indication if the coupon dates are adjusted to the payment dates. the
        possible values are:   adjusted,   unadjusted.  if instrumentcode is defined,
        the value comes from the instrument reference data. in case of a user-defined
        instrument, the default value is 'adjusted'.
        :return: enum AdjustInterestToPaymentDate
        """
        return self._get_enum_parameter(
            AdjustInterestToPaymentDate, "adjustInterestToPaymentDate"
        )

    @adjust_interest_to_payment_date.setter
    def adjust_interest_to_payment_date(self, value):
        self._set_enum_parameter(
            AdjustInterestToPaymentDate, "adjustInterestToPaymentDate", value
        )

    @property
    def amortization_schedule(self):
        """
        The amortization schedule of the instrument. it contains the following
        information:   startdate,   enddate,   remainingnotional,
        amortizationfrequency,   amount,   amortizationtype.  optional. no default value
        applies.
        :return: list AmortizationItem
        """
        return self._get_list_parameter(AmortizationItem, "amortizationSchedule")

    @amortization_schedule.setter
    def amortization_schedule(self, value):
        self._set_list_parameter(AmortizationItem, "amortizationSchedule", value)

    @property
    def direction(self):
        """
        The indication whether the cash flows of the instrument's leg are paid or
        received. the possible values are:   paid: the cash flows are paid to the
        counterparty,   received: the cash flows are received from the counterparty.  no
        default value applies.
        :return: enum Direction
        """
        return self._get_enum_parameter(Direction, "direction")

    @direction.setter
    def direction(self, value):
        self._set_enum_parameter(Direction, "direction", value)

    @property
    def index_average_method(self):
        """
        The value of the average index calculation method. the possible values are:
        compoundedactual,      dailycompoundedaverage,      compoundedaveragerate,
        arithmeticaverage
        :return: enum IndexAverageMethod
        """
        return self._get_enum_parameter(IndexAverageMethod, "indexAverageMethod")

    @index_average_method.setter
    def index_average_method(self, value):
        self._set_enum_parameter(IndexAverageMethod, "indexAverageMethod", value)

    @property
    def index_compounding_method(self):
        """
        The method how the interest rate is calculated from the reset floating rates
        when the reset frequency is higher than the interest payment frequency (e.g.
        daily index reset with quarterly interest payments). the possible values are:
        compounded, average, constant, adjustedcompounded, mexicancompounded. if
        instrumentcode is defined, the value comes from the instrument reference data.
        in case of a user-defined instrument, the default value is 'constant'.
        :return: enum IndexCompoundingMethod
        """
        return self._get_enum_parameter(
            IndexCompoundingMethod, "indexCompoundingMethod"
        )

    @index_compounding_method.setter
    def index_compounding_method(self, value):
        self._set_enum_parameter(
            IndexCompoundingMethod, "indexCompoundingMethod", value
        )

    @property
    def index_observation_method(self):
        """
        (rfr) method for determining the accrual observation period. the possible values
        are:      lookback: use the interest period for both rate accrual and interest
        payment.      periodshift: use the observation period for both rate accrual and
        interest payment.      mixed: use the observation period for rate accrual and
        the interest period for interest payment.
        :return: enum IndexObservationMethod
        """
        return self._get_enum_parameter(
            IndexObservationMethod, "indexObservationMethod"
        )

    @index_observation_method.setter
    def index_observation_method(self, value):
        self._set_enum_parameter(
            IndexObservationMethod, "indexObservationMethod", value
        )

    @property
    def index_reset_frequency(self):
        """
        The reset frequency for the floating instrument (e.g. annual, semiannual).   the
        default value is interestpaymentfrequency.
        :return: enum Frequency
        """
        return self._get_enum_parameter(Frequency, "indexResetFrequency")

    @index_reset_frequency.setter
    def index_reset_frequency(self, value):
        self._set_enum_parameter(Frequency, "indexResetFrequency", value)

    @property
    def index_reset_type(self):
        """
        A type indicating if the floating rate index is reset before the coupon period
        starts or at the end of the coupon period. the possible values are: inadvance:
        resets the index before the start of the interest period, inarrears: resets the
        index at the end of the interest period. the default value is 'inadvance'.
        :return: enum IndexResetType
        """
        return self._get_enum_parameter(IndexResetType, "indexResetType")

    @index_reset_type.setter
    def index_reset_type(self, value):
        self._set_enum_parameter(IndexResetType, "indexResetType", value)

    @property
    def index_spread_compounding_method(self):
        """
        The method defining how the computed float leg spread is applied to compounded
        rate. it applies only when indexcompoundingmethod= compounded. the possible
        values are:    isdacompounding,    nocompounding,    isdaflatcompounding.  the
        default value is 'isdacompounding'.
        :return: enum IndexSpreadCompoundingMethod
        """
        return self._get_enum_parameter(
            IndexSpreadCompoundingMethod, "indexSpreadCompoundingMethod"
        )

    @index_spread_compounding_method.setter
    def index_spread_compounding_method(self, value):
        self._set_enum_parameter(
            IndexSpreadCompoundingMethod, "indexSpreadCompoundingMethod", value
        )

    @property
    def interest_calculation_convention(self):
        """
        The day count basis method convention used to calculate the interest payments.
        optional. defaults to moneymarket. if instrumentcode is defined, the value comes
        from the instrument reference data.
        :return: enum InterestCalculationConvention
        """
        return self._get_enum_parameter(
            InterestCalculationConvention, "interestCalculationConvention"
        )

    @interest_calculation_convention.setter
    def interest_calculation_convention(self, value):
        self._set_enum_parameter(
            InterestCalculationConvention, "interestCalculationConvention", value
        )

    @property
    def interest_calculation_method(self):
        """
        The day count basis method used to calculate the interest payments(e.g.
        dcb_30_360, dcb_30_actual). the default value is selected based onnotionalccy.
        :return: enum DayCountBasis
        """
        return self._get_enum_parameter(DayCountBasis, "interestCalculationMethod")

    @interest_calculation_method.setter
    def interest_calculation_method(self, value):
        self._set_enum_parameter(DayCountBasis, "interestCalculationMethod", value)

    @property
    def interest_payment_frequency(self):
        """
        The frequency of the interest payment. either indexresetfrequency or
        interestpaymentfrequency must be provided (e.g. annual, semiannual).   the
        default value is indexresetfrequency.
        :return: enum Frequency
        """
        return self._get_enum_parameter(Frequency, "interestPaymentFrequency")

    @interest_payment_frequency.setter
    def interest_payment_frequency(self, value):
        self._set_enum_parameter(Frequency, "interestPaymentFrequency", value)

    @property
    def interest_type(self):
        """
        An indicator whether the instrument pays a fixed or floating interest. the
        possible values are: fixed, float. no default value applies.
        :return: enum InterestType
        """
        return self._get_enum_parameter(InterestType, "interestType")

    @interest_type.setter
    def interest_type(self, value):
        self._set_enum_parameter(InterestType, "interestType", value)

    @property
    def notional_exchange(self):
        """
        An indicator if the notional amount is exchanged and when it is exchanged. the
        possible values are:    none,    start,    end,    both,    endadjustment.   the
        default value is 'none'.
        :return: enum NotionalExchange
        """
        return self._get_enum_parameter(NotionalExchange, "notionalExchange")

    @notional_exchange.setter
    def notional_exchange(self, value):
        self._set_enum_parameter(NotionalExchange, "notionalExchange", value)

    @property
    def payment_business_day_convention(self):
        """
        The method to adjust dates to working days. the possible values are:
        previousbusinessday,    nextbusinessday,    modified following,    nomoving,
        bbswmodifiedfollowing.   if instrumentcode is defined, the value comes from the
        instrument reference data. in case of a user-defined instrument, the default
        value is'modifiedfollowing'.
        :return: enum BusinessDayConvention
        """
        return self._get_enum_parameter(
            BusinessDayConvention, "paymentBusinessDayConvention"
        )

    @payment_business_day_convention.setter
    def payment_business_day_convention(self, value):
        self._set_enum_parameter(
            BusinessDayConvention, "paymentBusinessDayConvention", value
        )

    @property
    def payment_roll_convention(self):
        """
        The method to adjust payment dates when they fall at the end of the month (e.g.
        28th of february, 30th, 31st). the possible values are:    last,    same,
        last28,    same28.   if instrumentcode is defined, the value comes from the
        instrument reference data. in case of a user-defined instrument, the default
        value is'same'.
        :return: enum DateRollingConvention
        """
        return self._get_enum_parameter(DateRollingConvention, "paymentRollConvention")

    @payment_roll_convention.setter
    def payment_roll_convention(self, value):
        self._set_enum_parameter(DateRollingConvention, "paymentRollConvention", value)

    @property
    def stub_rule(self):
        """
        The rule that defines whether coupon roll dates are aligned to the maturity or
        issue date. the possible values are:   issue,   maturity,   shortfirstprorata,
        shortfirstfull,   longfirstfull,   shortlastprorata.  the default value is
        'maturity'.
        :return: enum StubRule
        """
        return self._get_enum_parameter(StubRule, "stubRule")

    @stub_rule.setter
    def stub_rule(self, value):
        self._set_enum_parameter(StubRule, "stubRule", value)

    @property
    def cms_template(self):
        """
        A reference to a common swap contract that represents the underlying swap in
        case of a constant maturity swap contract (cms). example: eur_ab6e. no default
        value applies.
        :return: str
        """
        return self._get_parameter("cmsTemplate")

    @cms_template.setter
    def cms_template(self, value):
        self._set_parameter("cmsTemplate", value)

    @property
    def first_regular_payment_date(self):
        """
        The first regular interest payment date used for the odd first interest period.
        the value is expressed in iso 8601 format: yyyy-mm-ddt[hh]:[mm]:[ss]z (e.g.
        2021-01-01t00:00:00z). no default value applies.
        :return: str
        """
        return self._get_parameter("firstRegularPaymentDate")

    @first_regular_payment_date.setter
    def first_regular_payment_date(self, value):
        self._set_parameter("firstRegularPaymentDate", value)

    @property
    def fixed_rate_percent(self):
        """
        The interest rate of the instrument. the value is expressed in percentages.
        mandatory if no instrumentcode is defined. if instrumentcode is defined, the
        value comes from the instrument reference data.
        :return: float
        """
        return self._get_parameter("fixedRatePercent")

    @fixed_rate_percent.setter
    def fixed_rate_percent(self, value):
        self._set_parameter("fixedRatePercent", value)

    @property
    def floor_strike_percent(self):
        """
        The contractual strike rate of the floor. the value is expressed in percentages.
        if this parameter is set, the floor will apply to the leg with the same
        parameters set in the swaplegdefinition (e.g.maturity, frequency, index,
        discounting rule). no default value applies.
        :return: float
        """
        return self._get_parameter("floorStrikePercent")

    @floor_strike_percent.setter
    def floor_strike_percent(self, value):
        self._set_parameter("floorStrikePercent", value)

    @property
    def index_fixing_lag(self):
        """
        The number of working daysbetween the fixing date and the start of the coupon
        period ('inadvance') or the end of the coupon period ('inarrears'). the
        inadvance/inarrears mode is set in the indexresettype parameter. the default
        value is the fixing lag associated to the index defined/determined by default on
        the floating instrument.
        :return: int
        """
        return self._get_parameter("indexFixingLag")

    @index_fixing_lag.setter
    def index_fixing_lag(self, value):
        self._set_parameter("indexFixingLag", value)

    @property
    def index_fixing_ric(self):
        """
        The ric that carries the fixing value if the instrument has a floating interest.
        optional. mandatory for floating rate instruments if no instrumentcode is
        defined. if instrumentcode is defined, the value comes from the instrument
        reference data. no default value applies.
        :return: str
        """
        return self._get_parameter("indexFixingRic")

    @index_fixing_ric.setter
    def index_fixing_ric(self, value):
        self._set_parameter("indexFixingRic", value)

    @property
    def index_name(self):
        """
        The name of the floating rate index (e.g. 'euribor'). no default value applies.
        :return: str
        """
        return self._get_parameter("indexName")

    @index_name.setter
    def index_name(self, value):
        self._set_parameter("indexName", value)

    @property
    def index_tenor(self):
        """
        The period code indicating the maturity of the floating rate index. the default
        value is the tenor equivalent toindexresetfrequency or interestpaymentfrequency.
        :return: str
        """
        return self._get_parameter("indexTenor")

    @index_tenor.setter
    def index_tenor(self, value):
        self._set_parameter("indexTenor", value)

    @property
    def instrument_tag(self):
        """
        :return: str
        """
        return self._get_parameter("instrumentTag")

    @instrument_tag.setter
    def instrument_tag(self, value):
        self._set_parameter("instrumentTag", value)

    @property
    def interest_payment_delay(self):
        """
        The number of working days between the end of the interest accrual period and
        the interest payment date. by default, no delay (0) is applied.
        :return: int
        """
        return self._get_parameter("interestPaymentDelay")

    @interest_payment_delay.setter
    def interest_payment_delay(self, value):
        self._set_parameter("interestPaymentDelay", value)

    @property
    def last_regular_payment_date(self):
        """
        The last regular interest payment date used for the odd last interest period.
        the value is expressed in iso 8601 format: yyyy-mm-ddt[hh]:[mm]:[ss]z (e.g.
        2021-01-01t00:00:00z). no default value applies.
        :return: str
        """
        return self._get_parameter("lastRegularPaymentDate")

    @last_regular_payment_date.setter
    def last_regular_payment_date(self, value):
        self._set_parameter("lastRegularPaymentDate", value)

    @property
    def leg_tag(self):
        """
        A user-defined string to identify the direction of the leg: 'paid' or
        'received'. optional. no default value applies.
        :return: str
        """
        return self._get_parameter("legTag")

    @leg_tag.setter
    def leg_tag(self, value):
        self._set_parameter("legTag", value)

    @property
    def notional_amount(self):
        """
        The notional amount of the instrument's leg. the default value is '1,000,000'.
        :return: float
        """
        return self._get_parameter("notionalAmount")

    @notional_amount.setter
    def notional_amount(self, value):
        self._set_parameter("notionalAmount", value)

    @property
    def notional_ccy(self):
        """
        The currency of the instrument's notional amount. the value is expressed in iso
        4217 alphabetical format (e.g. 'usd'). no default value applies.
        :return: str
        """
        return self._get_parameter("notionalCcy")

    @notional_ccy.setter
    def notional_ccy(self, value):
        self._set_parameter("notionalCcy", value)

    @property
    def payment_business_days(self):
        """
        A list of comma-separated calendar codes to adjust dates (e.g. 'emu' or 'usa').
        the default value is the calendar associated to the market conventions of the
        interestpaymentccy for the corresponding leg.
        :return: str
        """
        return self._get_parameter("paymentBusinessDays")

    @payment_business_days.setter
    def payment_business_days(self, value):
        self._set_parameter("paymentBusinessDays", value)

    @property
    def spread_bp(self):
        """
        The interest spread in basis points that is added to the floating rate index
        value. optional. if instrumentcode is defined, the value comes from the
        instrument reference data. in case of a user-defined instrument, the default
        value is '0'.
        :return: float
        """
        return self._get_parameter("spreadBp")

    @spread_bp.setter
    def spread_bp(self, value):
        self._set_parameter("spreadBp", value)

    @property
    def upfront_amount(self):
        """
        The amount which represents the net present value of the swap. it is computed as
        [(100  dirtypricepercent / 100) x notionalamount]. the value is expressed in
        upfrontamountccy. by default, no payment (0) applies.
        :return: float
        """
        return self._get_parameter("upfrontAmount")

    @upfront_amount.setter
    def upfront_amount(self, value):
        self._set_parameter("upfrontAmount", value)



import allure
import pytest
from IPython.core.display import Image

from refinitiv.data.errors import RDError
from refinitiv.data.content import news
from tests.integration.constants_list import HttpStatusCode, HttpReason
from tests.integration.content.news.conftest import (
    check_file_is_saved,
)
from tests.integration.helpers import (
    check_response_status,
    get_async_response_from_definition, check_extended_params_were_sent_in_request,
)


@allure.suite("Content object - News Images")
@allure.feature("Content object - News Images")
@allure.severity(allure.severity_level.CRITICAL)
class TestNewsImages:
    @allure.title("Get News Images object and check data")
    @pytest.mark.caseid("51083859")
    @pytest.mark.parametrize(
        "image_id,width,height,expected_filename,expected_size",
        [
            (
                "22T112537Z_1_MQ4_RTRLXPP_2_LYNXPACKAGER__JPG",
                None,
                None,
                "2022-11-22T112537Z_1_MQ4_RTRLXPP_2_LYNXPACKAGER.JPG",
                38174,
            ),
            (
                "22T112537Z_1_MQ4_RTRLXPP_2_LYNXPACKAGER__JPG",
                100,
                70,
                "22T112537Z_1_MQ4_RTRLXPP_2_LYNXPACKAGER__JPG.jpeg",
                2241,
            ),
            (
                "tag:reuters__com,2023:newsml_LYNXMPEJ0F0CH:1___tag:reuters__com,2023:binary_LYNXMPEJ0F0CH-VIEWIMAGE",
                None,
                1100,
                "tag_reuters__com,2023_newsml_LYNXMPEJ0F0CH_1___tag_reuters__com,2023_binary_LYNXMPEJ0F0CH-VIEWIMAGE.jpeg",
                90226,
            ),
        ],
    )
    @pytest.mark.smoke
    def test_get_news_images_object_and_check_data(
        self,
        open_platform_session,
        image_id,
        width,
        height,
        expected_filename,
        expected_size,
    ):
        response = news.images.Definition(image_id, width, height).get_data()
        image = response.data.image

        assert isinstance(response.data.raw, dict)
        assert image.size == expected_size, f"Actual size: {image.size}"
        assert image.filename == expected_filename
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)

    @allure.title("Get News Images object and save image")
    @pytest.mark.caseid("C51083860")
    @pytest.mark.parametrize(
        "image_id,width,height,expected_filename",
        [
            (
                "22T112537Z_1_MQ4_RTRLXPP_2_LYNXPACKAGER__JPG",
                None,
                None,
                "2022-11-22T112537Z_1_MQ4_RTRLXPP_2_LYNXPACKAGER.JPG",
            ),
            (
                "22T112537Z_1_MQ4_RTRLXPP_2_LYNXPACKAGER__JPG",
                100,
                70,
                "22T112537Z_1_MQ4_RTRLXPP_2_LYNXPACKAGER__JPG.jpeg",
            ),
        ],
    )
    @pytest.mark.smoke
    def test_get_news_images_object_and_save_image(
        self, open_platform_session, image_id, width, height, expected_filename
    ):
        response = news.images.Definition(image_id, width, height).get_data()
        image = response.data.image
        image.save()

        check_file_is_saved(expected_filename)
        assert isinstance(image.show(), Image)

    @allure.title("Get News Images object async and save image")
    @pytest.mark.caseid("C51083861")
    @pytest.mark.parametrize(
        "image_id,path,expected_filename",
        [
            (
                "22T112537Z_1_MQ4_RTRLXPP_2_LYNXPACKAGER__JPG",
                "./images",
                "2022-11-22T112537Z_1_MQ4_RTRLXPP_2_LYNXPACKAGER.JPG",
            )
        ],
    )
    @pytest.mark.smoke
    async def test_get_news_images_object_async_and_save_image(
        self, open_platform_session, image_id, path, expected_filename
    ):
        response = await get_async_response_from_definition(
            news.images.Definition(image_id)
        )
        image = response.data.image
        image.save(path=path)

        check_file_is_saved(expected_filename, path)
        assert isinstance(image.show(), Image)

    @allure.title("Get News Images object with invalid image_id")
    @pytest.mark.caseid("C51083862")
    @pytest.mark.smoke
    def test_get_news_images_object_with_invalid_image_id(self, open_platform_session):
        with pytest.raises(RDError) as error:
            news.images.Definition("INVALID_IMAGE_ID").get_data()

        assert str(error.value) == "Error code 404 | Story not found"

    @allure.title("Get News Images object with extended params and check data")
    @pytest.mark.caseid("C53961917")
    @pytest.mark.parametrize(
        "image_id,width,height,extended_params,expected_filename,expected_size",
        [
            (
                "22T112537Z_1_MQ4_RTRLXPP_2_LYNXPACKAGER__JPG",
                400,
                200,
                {"width": 100, "height": 70},
                "22T112537Z_1_MQ4_RTRLXPP_2_LYNXPACKAGER__JPG.jpeg",
                2241,
            ),
        ],
    )
    @pytest.mark.smoke
    def test_get_news_images_object_with_extended_params(
        self,
        open_platform_session,
        image_id,
        width,
        height,
        extended_params,
        expected_filename,
        expected_size,
    ):
        response = news.images.Definition(image_id, width, height, extended_params).get_data()
        image = response.data.image

        assert isinstance(response.data.raw, dict)
        assert image.size == expected_size, f"Actual size: {image.size}"
        assert image.filename == expected_filename
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_extended_params_were_sent_in_request(response, extended_params)



from refinitiv.data.content.ipa.dates_and_calendars import add_periods
from refinitiv.data.content.ipa.dates_and_calendars.add_periods._add_periods_data_provider import (
    AddPeriodsResponseFactory,
)
from refinitiv.data.delivery._data._data_provider import ParsedData

MOCKED_ADD_PERIODS_DATA = ParsedData(
    {"http_status_code": 200, "http_reason": "OK"},
    {},
    [
        {
            "date": "2020-05-07",
            "holidays": [
                {
                    "date": "2020-04-28",
                    "names": [
                        {
                            "name": "Radunitsa '20",
                            "calendars": ["BAR"],
                            "countries": ["BLR"],
                        }
                    ],
                    "calendars": ["BAR"],
                },
                {
                    "date": "2020-04-29",
                    "names": [
                        {
                            "name": "Showa Day",
                            "calendars": ["JAP"],
                            "countries": ["JPN"],
                        }
                    ],
                    "calendars": ["JAP"],
                },
                {
                    "date": "2020-04-30",
                    "names": [
                        {
                            "name": "Buddha's Birthday '20",
                            "calendars": ["KOR"],
                            "countries": ["KOR"],
                        }
                    ],
                    "calendars": ["KOR"],
                },
                {
                    "date": "2020-05-01",
                    "names": [
                        {
                            "name": "Labour Day (Day 1)",
                            "calendars": ["BAR"],
                            "countries": ["BLR"],
                        },
                        {
                            "name": "Labor Day",
                            "calendars": ["KOR"],
                            "countries": ["KOR"],
                        },
                    ],
                    "calendars": ["BAR", "KOR"],
                },
                {
                    "date": "2020-05-02",
                    "names": [
                        {
                            "name": "Weekend",
                            "calendars": ["BAR", "KOR", "JAP", "USA"],
                            "countries": ["BLR", "KOR", "JPN", "USA"],
                        }
                    ],
                    "calendars": ["BAR", "KOR", "JAP", "USA"],
                },
                {
                    "date": "2020-05-03",
                    "names": [
                        {
                            "name": "Constitution Day",
                            "calendars": ["JAP"],
                            "countries": ["JPN"],
                        },
                        {
                            "name": "Weekend",
                            "calendars": ["BAR", "KOR", "USA"],
                            "countries": ["BLR", "KOR", "USA"],
                        },
                    ],
                    "calendars": ["BAR", "KOR", "JAP", "USA"],
                },
                {
                    "date": "2020-05-04",
                    "names": [
                        {
                            "name": "Greenery Day",
                            "calendars": ["JAP"],
                            "countries": ["JPN"],
                        }
                    ],
                    "calendars": ["JAP"],
                },
                {
                    "date": "2020-05-05",
                    "names": [
                        {
                            "name": "Children's Day",
                            "calendars": ["KOR", "JAP"],
                            "countries": ["KOR", "JPN"],
                        }
                    ],
                    "calendars": ["KOR", "JAP"],
                },
                {
                    "date": "2020-05-06",
                    "names": [
                        {
                            "name": "1. Constitution Day (Observed)",
                            "calendars": ["JAP"],
                            "countries": ["JPN"],
                        }
                    ],
                    "calendars": ["JAP"],
                },
            ],
            "tag": "my request",
        }
    ],
)

MOCKED_DATA_ERROR = ParsedData(
    {"http_status_code": 200, "http_reason": "OK"},
    {},
    [
        {
            "error": {
                "status": "Error",
                "message": "Business Calendar: invalid period.",
                "code": "IPA_library.ErrorBusinessCalendar_InvalidPeriod",
            },
            "tag": "my request",
        }
    ],
    ["IPA_library.ErrorBusinessCalendar_InvalidPeriod"],
    ["Business Calendar: invalid period."],
)


def test_add_periods():
    definition = add_periods.Definition(
        tag="my request",
        start_date="2020-04-24",
        period="4D",
        calendars=["BAR", "KOR", "JAP"],
        date_moving_convention=add_periods.DateMovingConvention.NEXT_BUSINESS_DAY,
        end_of_month_convention=add_periods.EndOfMonthConvention.LAST,
        holiday_outputs=[
            add_periods.HolidayOutputs.DATE,
            add_periods.HolidayOutputs.NAMES,
            add_periods.HolidayOutputs.CALENDARS,
            add_periods.HolidayOutputs.COUNTRIES,
        ],
    )
    assert definition.request_item.period == "4D"
    assert definition.request_item.tag == "my request"
    assert definition.request_item.start_date == "2020-04-24"
    assert definition.request_item.calendars == ["BAR", "KOR", "JAP"]
    assert (
        definition.request_item.end_of_month_convention
        == add_periods.EndOfMonthConvention.LAST
    )
    assert definition.request_item.holiday_outputs == [
        add_periods.HolidayOutputs.DATE,
        add_periods.HolidayOutputs.NAMES,
        add_periods.HolidayOutputs.CALENDARS,
        add_periods.HolidayOutputs.COUNTRIES,
    ]


def test_add_periods_create_success():
    response_factory = AddPeriodsResponseFactory()
    response = response_factory.create_success(MOCKED_ADD_PERIODS_DATA)
    assert not response.data.df.empty
    assert response.data.raw == MOCKED_ADD_PERIODS_DATA.content_data
    assert response.data.added_period.date == "2020-05-07"
    assert response.data.added_period.tag == "my request"
    assert response.data.added_period.holidays[0].calendars == ["BAR"]
    assert response.data.added_period.holidays[0].names[0].countries == ["BLR"]


def test_add_periods_create_fail():
    response_factory = AddPeriodsResponseFactory()
    response = response_factory.create_fail(MOCKED_DATA_ERROR)
    assert response.data.raw == MOCKED_DATA_ERROR.content_data
    assert len(response.errors)



__all__ = (
    "_bond_curves",
    "forward_curves",
    "zc_curve_definitions",
    "zc_curves",
    "_cross_currency_curves",
)

from . import _bond_curves
from . import forward_curves
from . import zc_curve_definitions
from . import zc_curves
from . import _cross_currency_curves



import allure
import pytest

from refinitiv.data.content import pricing
from refinitiv.data._content_type import ContentType
from refinitiv.data.delivery._stream import stream_cxn_cache
from tests.integration.content.chain.conftest import (
    open_desktop_session_for_streaming_chain,
    open_deployed_session_for_streaming_chain,
    create_chain_stream,
    create_chain_stream_with_callbacks,
)

CONSTITUENT_ERROR_MESSAGE = "The record could not be found"


@allure.suite("Content object - Streaming Chain")
@allure.feature("Content object - Streaming Chain")
@allure.severity(allure.severity_level.CRITICAL)
class TestStreamingChain:
    @allure.title("Open StreamingPrices using streaming chain and get results")
    @pytest.mark.caseid("33534326")
    @pytest.mark.parametrize("name,expected_fields", [("0#.FCHI", ["ASK", "BID"])])
    def test_open_streaming_prices_using_streaming_chain(
        self, open_desktop_session, name, expected_fields
    ):
        stream = create_chain_stream(name)
        stream.open()
        constituents_list = stream.constituents

        streaming_prices = pricing.Definition(
            universe=constituents_list, fields=expected_fields
        ).get_stream()
        streaming_prices.open()
        snapshot = streaming_prices.get_snapshot()
        field_names = snapshot.columns[1:].tolist()
        instrument_names = snapshot["Instrument"].values
        stream.close()
        streaming_prices.close()

        assert instrument_names == constituents_list, (
            f"Instrument names retrieved from StreamingPrice {instrument_names}"
            f" are not equal to constituents retrieved from StreamingChain {constituents_list}"
        )
        assert (
            field_names.sort() == expected_fields.sort()
        ), f"Field names {field_names} are not equal to requested."

    @allure.title("Open a Streaming Chain with a valid name and  get constituents list")
    @pytest.mark.caseid("33533976")
    @pytest.mark.parametrize("name", ["0#.DJI"])
    @pytest.mark.smoke
    def test_open_streaming_chain_with_valid_name(self, open_desktop_session, name):
        stream = create_chain_stream(name)
        stream.open(with_updates=True)
        constituents_list = stream.constituents
        assert len(constituents_list) > 0, f"The empty list received in response"
        assert (
            CONSTITUENT_ERROR_MESSAGE not in constituents_list
        ), f"Value {CONSTITUENT_ERROR_MESSAGE} found in list of constituents {constituents_list}"
        assert stream.is_chain, "is_chain is not true"
        stream.close()

    @allure.title("Open a Streaming Chain with INVALID name")
    @pytest.mark.caseid("33534354")
    @pytest.mark.parametrize("name", ["test_invalid_name"])
    def test_open_streaming_chain_with_invalid_name(self, open_desktop_session, name):
        triggered_events = []
        stream = create_chain_stream_with_callbacks(
            name=name, triggered_events=triggered_events
        )
        stream.open(with_updates=True)
        constituents_list = stream.constituents
        stream.close()
        assert (
            len(constituents_list) == 0
        ), f"Received non-empty constituents list: {constituents_list}"
        assert "Error received" in triggered_events, f"Error event  was not triggered"

    @allure.title("Open a Streaming Chain with a closed session")
    @pytest.mark.caseid("33533977")
    @pytest.mark.parametrize("name", ["0#.DJI"])
    def test_open_streaming_chain_when_session_is_not_opened(
        self, name, open_platform_session
    ):
        session = open_platform_session
        session.close()
        stream = create_chain_stream(name)
        with pytest.raises(AssertionError, match="Session must be open"):
            stream.open(with_updates=True)

    @allure.title(
        "Open a Streaming Chain with summary_links property and receive an array of Summary links"
    )
    @pytest.mark.caseid("33534056")
    @pytest.mark.parametrize("name", ["0#.DJI"])
    def test_open_streaming_chain_with_summary_links(self, open_desktop_session, name):
        stream = create_chain_stream(name)
        stream.open(with_updates=True)
        summary_links = stream.summary_links
        stream.close()
        assert ".DJI" in summary_links

    @allure.title("Open a StreamingChain with event handler functions")
    @pytest.mark.caseid("33536578")
    @pytest.mark.parametrize(
        "name,open_custom_session,service",
        [
            ("0#.DJI", open_deployed_session_for_streaming_chain, "ELEKTRON_DD"),
            ("0#.DJI", open_desktop_session_for_streaming_chain, None),
        ],
    )
    def test_open_streaming_chain_with_event_handlers(
        self, open_custom_session, name, service
    ):
        session = open_custom_session()
        triggered_events = []
        expected_events = [
            "Add performed",
            "Complete performed",
            "Remove performed",
            "Update performed",
        ]
        stream = create_chain_stream_with_callbacks(
            name, triggered_events, service=service
        )
        stream.open(with_updates=True)

        stream._stream._remove_constituent(1, "TEST remove")
        stream._stream._update_constituent(3, "OLD", "TEST update")
        constituents_list = stream.constituents
        stream.close()
        session.close()

        is_cxn_alive = stream_cxn_cache.is_cxn_alive(
            session, ContentType.STREAMING_PRICING
        )

        assert not is_cxn_alive, f"Connection is alive"
        assert not stream._stream.is_opened, f"Stream is not closed"

        assert len(constituents_list) > 0, f"The empty list received in response"
        for event in expected_events:
            assert (
                event in triggered_events
            ), f"found event '{event}' not handled by handlers or not triggered"
        assert (
            triggered_events.count(expected_events[1]) == 1
        ), f"Callback on_complete called more than once: {triggered_events.count(expected_events[1])}"

    @allure.title("Open and close a Streaming Chain with context manager")
    @pytest.mark.caseid("C44023026")
    @pytest.mark.parametrize("name", ["0#.DJI"])
    @pytest.mark.smoke
    def test_open_streaming_chain_with_context_manager(
        self, open_desktop_session, name
    ):
        with create_chain_stream(name) as chain:
            assert chain._stream.is_open
        assert chain._stream.is_close

    @allure.title("Open and close a Streaming Chain with asynchronous context manager")
    @pytest.mark.caseid("C44023027")
    @pytest.mark.parametrize("name", ["0#.DJI"])
    @pytest.mark.smoke
    async def test_open_streaming_chain_with_async_context_manager(
        self, open_desktop_session_async, name
    ):
        async with create_chain_stream(name) as chain:
            assert chain._stream.is_open
        assert chain._stream.is_close



from refinitiv.data._qpl import FxSwapPoints, TenorTypes, TenorBidAsk


def test_fx_swap_points_complex():
    fx_swap_points = FxSwapPoints(
        additional_tenor_types=[TenorTypes.LONG, TenorTypes.ODD],
        source="D3",
        overrides=[
            TenorBidAsk(tenor="1M", bid=50, ask=60),
            TenorBidAsk(tenor="2M", bid=90),
        ],
    )

    assert fx_swap_points.overrides == [
        {"ask": 60, "bid": 50, "tenor": "1M"},
        {"bid": 90, "tenor": "2M"},
    ]
    assert fx_swap_points.source == "D3"
    assert fx_swap_points.additional_tenor_types == ["Long", "Odd"]

    assert fx_swap_points.get_dict() == {
        "additionalTenorTypes": ["Long", "Odd"],
        "overrides": [
            {"ask": 60, "bid": 50, "tenor": "1M"},
            {"bid": 90, "tenor": "2M"},
        ],
        "source": "D3",
    }


def test_fx_swap_points_primitive():
    fx_swap_points = FxSwapPoints(
        additional_tenor_types=["Long", "Odd"],
        source="D3",
        overrides=[{"ask": 60, "bid": 50, "tenor": "1M"}, {"bid": 90, "tenor": "2M"}],
    )

    assert fx_swap_points.overrides == [
        {"ask": 60, "bid": 50, "tenor": "1M"},
        {"bid": 90, "tenor": "2M"},
    ]
    assert fx_swap_points.source == "D3"
    assert fx_swap_points.additional_tenor_types == ["Long", "Odd"]

    assert fx_swap_points.get_dict() == {
        "additionalTenorTypes": ["Long", "Odd"],
        "overrides": [
            {"ask": 60, "bid": 50, "tenor": "1M"},
            {"bid": 90, "tenor": "2M"},
        ],
        "source": "D3",
    }


def test_tenor_types():
    assert str(TenorTypes.ODD) == "Odd"
    assert str(TenorTypes.LONG) == "Long"
    assert str(TenorTypes.IMM) == "IMM"
    assert str(TenorTypes.BEGINNING_OF_MONTH) == "BeginningOfMonth"
    assert str(TenorTypes.END_OF_MONTH) == "EndOfMonth"



from typing import Optional, TYPE_CHECKING

from ._shift_definition import ShiftDefinition
from ...._object_definition import ObjectDefinition


if TYPE_CHECKING:
    from ......_types import OptStr


class FxShiftScenario(ObjectDefinition):
    """
    Generates the Cross Currency curves for the definitions provided

    Parameters
    ----------
    fx_curve_shift : ShiftDefinition, optional
        Collection of shift parameters tenor. "all" selector supported as well.
    shift_tag : str, optional
        User defined string to identify the shift scenario tag. it can be used to link
        output curve to the shift scenario. only alphabetic, numeric and '- _.#=@'
        characters are supported. optional.
    """

    def __init__(
        self,
        fx_curve_shift: Optional[ShiftDefinition] = None,
        shift_tag: "OptStr" = None,
    ) -> None:
        super().__init__()
        self.fx_curve_shift = fx_curve_shift
        self.shift_tag = shift_tag

    @property
    def fx_curve_shift(self):
        """
        Collection of shift parameters tenor. "all" selector supported as well.
        :return: object ShiftDefinition
        """
        return self._get_object_parameter(ShiftDefinition, "fxCurveShift")

    @fx_curve_shift.setter
    def fx_curve_shift(self, value):
        self._set_object_parameter(ShiftDefinition, "fxCurveShift", value)

    @property
    def shift_tag(self):
        """
        User defined string to identify the shift scenario tag. it can be used to link
        output curve to the shift scenario. only alphabetic, numeric and '- _.#=@'
        characters are supported. optional.
        :return: str
        """
        return self._get_parameter("shiftTag")

    @shift_tag.setter
    def shift_tag(self, value):
        self._set_parameter("shiftTag", value)



import enum


def check_response_data_fields(response, expected_fields):
    for data_object in response.data.raw:
        actual_fields = {header["name"] for header in data_object["headers"]}
        assert set(expected_fields).issubset(
            actual_fields
        ), f"Not all requested fields are present in universe {data_object['universe']['ric']}"


def check_response_data_event_type(response, event_type):
    if isinstance(event_type, enum.Enum):
        event_type = event_type.value
    assert all(
        s == event_type for s in response.data.df["EVENT_TYPE"]
    ), f"Data contains event types that weren't requested: {list(response.data.df['EVENT_TYPE'])}"



from refinitiv.data.content.ipa.curves._cross_currency_curves.triangulate_definitions import (
    search,
)
from refinitiv.data.content.ipa.curves._cross_currency_curves.triangulate_definitions._data_provider import (
    TriangulateDefinitionsData,
)
from tests.unit.conftest import StubResponse, StubSession


def test_search_definition():
    # given
    response = StubResponse(
        {
            "data": [
                {
                    "directCurveDefinitions": [
                        {
                            "baseCurrency": "EUR",
                            "baseIndexName": "EURIBOR",
                            "name": "EUR CHF FxCross",
                            "quotedCurrency": "CHF",
                            "quotedIndexName": "LIBOR",
                            "source": "Refinitiv",
                            "isNonDeliverable": False,
                            "mainConstituentAssetClass": "Swap",
                            "riskType": "CrossCurrency",
                            "id": "1146241a-e32f-45f5-9309-02c959e97b96",
                            "isFallbackForFxCurveDefinition": True,
                        }
                    ],
                    "indirectCurveDefinitions": [
                        {
                            "crossCurrencyDefinitions": [
                                {
                                    "baseCurrency": "EUR",
                                    "baseIndexName": "ESTR",
                                    "name": "Name of the Curve rtf",
                                    "quotedCurrency": "USD",
                                    "quotedIndexName": "SOFR",
                                    "source": "SourceName",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "FxForward",
                                    "riskType": "CrossCurrency",
                                    "id": "fb633ff4-8c62-4c62-86e4-54637a4f11a3",
                                    "isFallbackForFxCurveDefinition": False,
                                },
                                {
                                    "baseCurrency": "USD",
                                    "baseIndexName": "LIBOR",
                                    "name": "USD CHF FxCross",
                                    "quotedCurrency": "CHF",
                                    "quotedIndexName": "LIBOR",
                                    "source": "Refinitiv",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "Swap",
                                    "riskType": "CrossCurrency",
                                    "id": "3475e008-de92-4b0c-a60c-6184d3c9afa8",
                                    "isFallbackForFxCurveDefinition": True,
                                },
                            ]
                        },
                        {
                            "crossCurrencyDefinitions": [
                                {
                                    "baseCurrency": "EUR",
                                    "baseIndexName": "ESTR",
                                    "name": "Name of the Curve rtf",
                                    "quotedCurrency": "USD",
                                    "quotedIndexName": "SOFR",
                                    "source": "SourceName",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "FxForward",
                                    "riskType": "CrossCurrency",
                                    "id": "fb633ff4-8c62-4c62-86e4-54637a4f11a3",
                                    "isFallbackForFxCurveDefinition": False,
                                },
                                {
                                    "baseCurrency": "USD",
                                    "baseIndexName": "SOFR",
                                    "name": "USD SOFR/CHF SARON FxCross",
                                    "quotedCurrency": "CHF",
                                    "quotedIndexName": "SARON",
                                    "source": "Refinitiv",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "Swap",
                                    "riskType": "CrossCurrency",
                                    "id": "6e07c9a4-0b99-47f1-b549-0a6e64081c9e",
                                    "isFallbackForFxCurveDefinition": False,
                                },
                            ]
                        },
                        {
                            "crossCurrencyDefinitions": [
                                {
                                    "baseCurrency": "EUR",
                                    "baseIndexName": "ESTR",
                                    "name": "Name of the Curve rtfrr",
                                    "quotedCurrency": "USD",
                                    "quotedIndexName": "SOFR",
                                    "source": "SourceName",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "FxForward",
                                    "riskType": "CrossCurrency",
                                    "id": "1f7c7066-4042-407d-847f-f632cd1cf73f",
                                    "isFallbackForFxCurveDefinition": False,
                                },
                                {
                                    "baseCurrency": "USD",
                                    "baseIndexName": "LIBOR",
                                    "name": "USD CHF FxCross",
                                    "quotedCurrency": "CHF",
                                    "quotedIndexName": "LIBOR",
                                    "source": "Refinitiv",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "Swap",
                                    "riskType": "CrossCurrency",
                                    "id": "3475e008-de92-4b0c-a60c-6184d3c9afa8",
                                    "isFallbackForFxCurveDefinition": True,
                                },
                            ]
                        },
                        {
                            "crossCurrencyDefinitions": [
                                {
                                    "baseCurrency": "EUR",
                                    "baseIndexName": "ESTR",
                                    "name": "Name of the Curve rtfrr",
                                    "quotedCurrency": "USD",
                                    "quotedIndexName": "SOFR",
                                    "source": "SourceName",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "FxForward",
                                    "riskType": "CrossCurrency",
                                    "id": "1f7c7066-4042-407d-847f-f632cd1cf73f",
                                    "isFallbackForFxCurveDefinition": False,
                                },
                                {
                                    "baseCurrency": "USD",
                                    "baseIndexName": "SOFR",
                                    "name": "USD SOFR/CHF SARON FxCross",
                                    "quotedCurrency": "CHF",
                                    "quotedIndexName": "SARON",
                                    "source": "Refinitiv",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "Swap",
                                    "riskType": "CrossCurrency",
                                    "id": "6e07c9a4-0b99-47f1-b549-0a6e64081c9e",
                                    "isFallbackForFxCurveDefinition": False,
                                },
                            ]
                        },
                        {
                            "crossCurrencyDefinitions": [
                                {
                                    "baseCurrency": "EUR",
                                    "baseIndexName": "ESTR",
                                    "name": "Name of the Curve rtf77rr",
                                    "quotedCurrency": "USD",
                                    "quotedIndexName": "SOFR",
                                    "source": "SourceName",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "FxForward",
                                    "riskType": "CrossCurrency",
                                    "id": "37eb1c0b-0bb0-4305-a4d8-2bb4a523d549",
                                    "isFallbackForFxCurveDefinition": False,
                                },
                                {
                                    "baseCurrency": "USD",
                                    "baseIndexName": "LIBOR",
                                    "name": "USD CHF FxCross",
                                    "quotedCurrency": "CHF",
                                    "quotedIndexName": "LIBOR",
                                    "source": "Refinitiv",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "Swap",
                                    "riskType": "CrossCurrency",
                                    "id": "3475e008-de92-4b0c-a60c-6184d3c9afa8",
                                    "isFallbackForFxCurveDefinition": True,
                                },
                            ]
                        },
                        {
                            "crossCurrencyDefinitions": [
                                {
                                    "baseCurrency": "EUR",
                                    "baseIndexName": "ESTR",
                                    "name": "Name of the Curve rtf77rr",
                                    "quotedCurrency": "USD",
                                    "quotedIndexName": "SOFR",
                                    "source": "SourceName",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "FxForward",
                                    "riskType": "CrossCurrency",
                                    "id": "37eb1c0b-0bb0-4305-a4d8-2bb4a523d549",
                                    "isFallbackForFxCurveDefinition": False,
                                },
                                {
                                    "baseCurrency": "USD",
                                    "baseIndexName": "SOFR",
                                    "name": "USD SOFR/CHF SARON FxCross",
                                    "quotedCurrency": "CHF",
                                    "quotedIndexName": "SARON",
                                    "source": "Refinitiv",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "Swap",
                                    "riskType": "CrossCurrency",
                                    "id": "6e07c9a4-0b99-47f1-b549-0a6e64081c9e",
                                    "isFallbackForFxCurveDefinition": False,
                                },
                            ]
                        },
                        {
                            "crossCurrencyDefinitions": [
                                {
                                    "baseCurrency": "EUR",
                                    "baseIndexName": "ESTR",
                                    "name": "Name of the Curve223",
                                    "quotedCurrency": "USD",
                                    "quotedIndexName": "SOFR",
                                    "source": "SourceName",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "Swap",
                                    "riskType": "CrossCurrency",
                                    "id": "02b711e1-c221-4044-b2c3-a4c2ca886a60",
                                    "isFallbackForFxCurveDefinition": False,
                                },
                                {
                                    "baseCurrency": "USD",
                                    "baseIndexName": "LIBOR",
                                    "name": "USD CHF FxCross",
                                    "quotedCurrency": "CHF",
                                    "quotedIndexName": "LIBOR",
                                    "source": "Refinitiv",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "Swap",
                                    "riskType": "CrossCurrency",
                                    "id": "3475e008-de92-4b0c-a60c-6184d3c9afa8",
                                    "isFallbackForFxCurveDefinition": True,
                                },
                            ]
                        },
                        {
                            "crossCurrencyDefinitions": [
                                {
                                    "baseCurrency": "EUR",
                                    "baseIndexName": "ESTR",
                                    "name": "Name of the Curve223",
                                    "quotedCurrency": "USD",
                                    "quotedIndexName": "SOFR",
                                    "source": "SourceName",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "Swap",
                                    "riskType": "CrossCurrency",
                                    "id": "02b711e1-c221-4044-b2c3-a4c2ca886a60",
                                    "isFallbackForFxCurveDefinition": False,
                                },
                                {
                                    "baseCurrency": "USD",
                                    "baseIndexName": "SOFR",
                                    "name": "USD SOFR/CHF SARON FxCross",
                                    "quotedCurrency": "CHF",
                                    "quotedIndexName": "SARON",
                                    "source": "Refinitiv",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "Swap",
                                    "riskType": "CrossCurrency",
                                    "id": "6e07c9a4-0b99-47f1-b549-0a6e64081c9e",
                                    "isFallbackForFxCurveDefinition": False,
                                },
                            ]
                        },
                        {
                            "crossCurrencyDefinitions": [
                                {
                                    "baseCurrency": "EUR",
                                    "baseIndexName": "ESTR",
                                    "name": "Name of the Curve78874",
                                    "quotedCurrency": "USD",
                                    "quotedIndexName": "SOFR",
                                    "source": "SourceName",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "FxForward",
                                    "riskType": "CrossCurrency",
                                    "id": "f02d9b4b-ec08-4cd3-9193-c325c3787079",
                                    "isFallbackForFxCurveDefinition": False,
                                },
                                {
                                    "baseCurrency": "USD",
                                    "baseIndexName": "LIBOR",
                                    "name": "USD CHF FxCross",
                                    "quotedCurrency": "CHF",
                                    "quotedIndexName": "LIBOR",
                                    "source": "Refinitiv",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "Swap",
                                    "riskType": "CrossCurrency",
                                    "id": "3475e008-de92-4b0c-a60c-6184d3c9afa8",
                                    "isFallbackForFxCurveDefinition": True,
                                },
                            ]
                        },
                        {
                            "crossCurrencyDefinitions": [
                                {
                                    "baseCurrency": "EUR",
                                    "baseIndexName": "ESTR",
                                    "name": "Name of the Curve78874",
                                    "quotedCurrency": "USD",
                                    "quotedIndexName": "SOFR",
                                    "source": "SourceName",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "FxForward",
                                    "riskType": "CrossCurrency",
                                    "id": "f02d9b4b-ec08-4cd3-9193-c325c3787079",
                                    "isFallbackForFxCurveDefinition": False,
                                },
                                {
                                    "baseCurrency": "USD",
                                    "baseIndexName": "SOFR",
                                    "name": "USD SOFR/CHF SARON FxCross",
                                    "quotedCurrency": "CHF",
                                    "quotedIndexName": "SARON",
                                    "source": "Refinitiv",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "Swap",
                                    "riskType": "CrossCurrency",
                                    "id": "6e07c9a4-0b99-47f1-b549-0a6e64081c9e",
                                    "isFallbackForFxCurveDefinition": False,
                                },
                            ]
                        },
                        {
                            "crossCurrencyDefinitions": [
                                {
                                    "baseCurrency": "EUR",
                                    "baseIndexName": "EURIBOR",
                                    "name": "EUR USD FxCross",
                                    "quotedCurrency": "USD",
                                    "quotedIndexName": "LIBOR",
                                    "source": "Refinitiv",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "Swap",
                                    "riskType": "CrossCurrency",
                                    "id": "334b89f6-e272-4900-ad1e-c023b516ae13",
                                    "isFallbackForFxCurveDefinition": True,
                                },
                                {
                                    "baseCurrency": "USD",
                                    "baseIndexName": "LIBOR",
                                    "name": "USD CHF FxCross",
                                    "quotedCurrency": "CHF",
                                    "quotedIndexName": "LIBOR",
                                    "source": "Refinitiv",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "Swap",
                                    "riskType": "CrossCurrency",
                                    "id": "3475e008-de92-4b0c-a60c-6184d3c9afa8",
                                    "isFallbackForFxCurveDefinition": True,
                                },
                            ]
                        },
                        {
                            "crossCurrencyDefinitions": [
                                {
                                    "baseCurrency": "EUR",
                                    "baseIndexName": "EURIBOR",
                                    "name": "EUR USD FxCross",
                                    "quotedCurrency": "USD",
                                    "quotedIndexName": "LIBOR",
                                    "source": "Refinitiv",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "Swap",
                                    "riskType": "CrossCurrency",
                                    "id": "334b89f6-e272-4900-ad1e-c023b516ae13",
                                    "isFallbackForFxCurveDefinition": True,
                                },
                                {
                                    "baseCurrency": "USD",
                                    "baseIndexName": "SOFR",
                                    "name": "USD SOFR/CHF SARON FxCross",
                                    "quotedCurrency": "CHF",
                                    "quotedIndexName": "SARON",
                                    "source": "Refinitiv",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "Swap",
                                    "riskType": "CrossCurrency",
                                    "id": "6e07c9a4-0b99-47f1-b549-0a6e64081c9e",
                                    "isFallbackForFxCurveDefinition": False,
                                },
                            ]
                        },
                        {
                            "crossCurrencyDefinitions": [
                                {
                                    "baseCurrency": "EUR",
                                    "baseIndexName": "ESTR",
                                    "name": "EUR ESTR/USD SOFR FxCross",
                                    "quotedCurrency": "USD",
                                    "quotedIndexName": "SOFR",
                                    "source": "Refinitiv",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "Swap",
                                    "riskType": "CrossCurrency",
                                    "id": "c9f2e9fb-b04b-4140-8377-8b7e47391486",
                                    "isFallbackForFxCurveDefinition": False,
                                },
                                {
                                    "baseCurrency": "USD",
                                    "baseIndexName": "LIBOR",
                                    "name": "USD CHF FxCross",
                                    "quotedCurrency": "CHF",
                                    "quotedIndexName": "LIBOR",
                                    "source": "Refinitiv",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "Swap",
                                    "riskType": "CrossCurrency",
                                    "id": "3475e008-de92-4b0c-a60c-6184d3c9afa8",
                                    "isFallbackForFxCurveDefinition": True,
                                },
                            ]
                        },
                        {
                            "crossCurrencyDefinitions": [
                                {
                                    "baseCurrency": "EUR",
                                    "baseIndexName": "ESTR",
                                    "name": "EUR ESTR/USD SOFR FxCross",
                                    "quotedCurrency": "USD",
                                    "quotedIndexName": "SOFR",
                                    "source": "Refinitiv",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "Swap",
                                    "riskType": "CrossCurrency",
                                    "id": "c9f2e9fb-b04b-4140-8377-8b7e47391486",
                                    "isFallbackForFxCurveDefinition": False,
                                },
                                {
                                    "baseCurrency": "USD",
                                    "baseIndexName": "SOFR",
                                    "name": "USD SOFR/CHF SARON FxCross",
                                    "quotedCurrency": "CHF",
                                    "quotedIndexName": "SARON",
                                    "source": "Refinitiv",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "Swap",
                                    "riskType": "CrossCurrency",
                                    "id": "6e07c9a4-0b99-47f1-b549-0a6e64081c9e",
                                    "isFallbackForFxCurveDefinition": False,
                                },
                            ]
                        },
                        {
                            "crossCurrencyDefinitions": [
                                {
                                    "baseCurrency": "EUR",
                                    "baseIndexName": "EONIA",
                                    "name": "EUR EONIA/USD SOFR FxCross",
                                    "quotedCurrency": "USD",
                                    "quotedIndexName": "SOFR",
                                    "source": "Refinitiv",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "Swap",
                                    "riskType": "CrossCurrency",
                                    "id": "4246920e-fb65-4b62-aa59-fa492ae6b1c2",
                                    "isFallbackForFxCurveDefinition": False,
                                },
                                {
                                    "baseCurrency": "USD",
                                    "baseIndexName": "LIBOR",
                                    "name": "USD CHF FxCross",
                                    "quotedCurrency": "CHF",
                                    "quotedIndexName": "LIBOR",
                                    "source": "Refinitiv",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "Swap",
                                    "riskType": "CrossCurrency",
                                    "id": "3475e008-de92-4b0c-a60c-6184d3c9afa8",
                                    "isFallbackForFxCurveDefinition": True,
                                },
                            ]
                        },
                        {
                            "crossCurrencyDefinitions": [
                                {
                                    "baseCurrency": "EUR",
                                    "baseIndexName": "EONIA",
                                    "name": "EUR EONIA/USD SOFR FxCross",
                                    "quotedCurrency": "USD",
                                    "quotedIndexName": "SOFR",
                                    "source": "Refinitiv",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "Swap",
                                    "riskType": "CrossCurrency",
                                    "id": "4246920e-fb65-4b62-aa59-fa492ae6b1c2",
                                    "isFallbackForFxCurveDefinition": False,
                                },
                                {
                                    "baseCurrency": "USD",
                                    "baseIndexName": "SOFR",
                                    "name": "USD SOFR/CHF SARON FxCross",
                                    "quotedCurrency": "CHF",
                                    "quotedIndexName": "SARON",
                                    "source": "Refinitiv",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "Swap",
                                    "riskType": "CrossCurrency",
                                    "id": "6e07c9a4-0b99-47f1-b549-0a6e64081c9e",
                                    "isFallbackForFxCurveDefinition": False,
                                },
                            ]
                        },
                        {
                            "crossCurrencyDefinitions": [
                                {
                                    "baseCurrency": "EUR",
                                    "baseIndexName": "ESTR",
                                    "name": "Name of the Curve",
                                    "quotedCurrency": "USD",
                                    "quotedIndexName": "SOFR",
                                    "source": "SourceName",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "Swap",
                                    "riskType": "CrossCurrency",
                                    "id": "a41be363-5709-41d7-bf6d-4233cd815070",
                                    "isFallbackForFxCurveDefinition": False,
                                },
                                {
                                    "baseCurrency": "USD",
                                    "baseIndexName": "LIBOR",
                                    "name": "USD CHF FxCross",
                                    "quotedCurrency": "CHF",
                                    "quotedIndexName": "LIBOR",
                                    "source": "Refinitiv",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "Swap",
                                    "riskType": "CrossCurrency",
                                    "id": "3475e008-de92-4b0c-a60c-6184d3c9afa8",
                                    "isFallbackForFxCurveDefinition": True,
                                },
                            ]
                        },
                        {
                            "crossCurrencyDefinitions": [
                                {
                                    "baseCurrency": "EUR",
                                    "baseIndexName": "ESTR",
                                    "name": "Name of the Curve",
                                    "quotedCurrency": "USD",
                                    "quotedIndexName": "SOFR",
                                    "source": "SourceName",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "Swap",
                                    "riskType": "CrossCurrency",
                                    "id": "a41be363-5709-41d7-bf6d-4233cd815070",
                                    "isFallbackForFxCurveDefinition": False,
                                },
                                {
                                    "baseCurrency": "USD",
                                    "baseIndexName": "SOFR",
                                    "name": "USD SOFR/CHF SARON FxCross",
                                    "quotedCurrency": "CHF",
                                    "quotedIndexName": "SARON",
                                    "source": "Refinitiv",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "Swap",
                                    "riskType": "CrossCurrency",
                                    "id": "6e07c9a4-0b99-47f1-b549-0a6e64081c9e",
                                    "isFallbackForFxCurveDefinition": False,
                                },
                            ]
                        },
                        {
                            "crossCurrencyDefinitions": [
                                {
                                    "baseCurrency": "EUR",
                                    "baseIndexName": "ESTR",
                                    "name": "Name of the Curve78954",
                                    "quotedCurrency": "USD",
                                    "quotedIndexName": "SOFR",
                                    "source": "SourceName",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "FxForward",
                                    "riskType": "CrossCurrency",
                                    "id": "c0086a8a-3c3b-4adc-a217-8c95b93a4d8d",
                                    "isFallbackForFxCurveDefinition": False,
                                },
                                {
                                    "baseCurrency": "USD",
                                    "baseIndexName": "LIBOR",
                                    "name": "USD CHF FxCross",
                                    "quotedCurrency": "CHF",
                                    "quotedIndexName": "LIBOR",
                                    "source": "Refinitiv",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "Swap",
                                    "riskType": "CrossCurrency",
                                    "id": "3475e008-de92-4b0c-a60c-6184d3c9afa8",
                                    "isFallbackForFxCurveDefinition": True,
                                },
                            ]
                        },
                        {
                            "crossCurrencyDefinitions": [
                                {
                                    "baseCurrency": "EUR",
                                    "baseIndexName": "ESTR",
                                    "name": "Name of the Curve78954",
                                    "quotedCurrency": "USD",
                                    "quotedIndexName": "SOFR",
                                    "source": "SourceName",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "FxForward",
                                    "riskType": "CrossCurrency",
                                    "id": "c0086a8a-3c3b-4adc-a217-8c95b93a4d8d",
                                    "isFallbackForFxCurveDefinition": False,
                                },
                                {
                                    "baseCurrency": "USD",
                                    "baseIndexName": "SOFR",
                                    "name": "USD SOFR/CHF SARON FxCross",
                                    "quotedCurrency": "CHF",
                                    "quotedIndexName": "SARON",
                                    "source": "Refinitiv",
                                    "isNonDeliverable": False,
                                    "mainConstituentAssetClass": "Swap",
                                    "riskType": "CrossCurrency",
                                    "id": "6e07c9a4-0b99-47f1-b549-0a6e64081c9e",
                                    "isFallbackForFxCurveDefinition": False,
                                },
                            ]
                        },
                    ],
                }
            ]
        }
    )
    session = StubSession(is_open=True, response=response)
    definition = search.Definition(base_currency="EUR", quoted_currency="CHF")

    # when
    response = definition.get_data(session)

    # then
    assert response.data.raw
    assert isinstance(response.data, TriangulateDefinitionsData)



from enum import Enum, unique


@unique
class Seniority(Enum):
    SENIOR_NON_PREFERRED = "SeniorNonPreferred"
    SENIOR_PREFERRED = "SeniorPreferred"
    SUBORDINATE_UNSECURED = "SubordinateUnsecured"



from typing import TYPE_CHECKING

from .. import content

from ..content.symbol_conversion._definition import DEFAULT_SCOPE
from ..content.symbol_conversion._symbol_type import SYMBOL_TYPE_VALUES

if TYPE_CHECKING:
    from ..content.symbol_conversion._symbol_type import OptSymbolTypes
    from ..content.symbol_conversion._symbol_type import SymbolTypes
    from .._types import StrStrings
    from ..content.symbol_conversion._symbol_type import OptAssetState
    from ..content.symbol_conversion._symbol_type import OptCountryCode
    from ..content.symbol_conversion._symbol_type import OptAssetClass


def convert_symbols(
    symbols: "StrStrings",
    from_symbol_type: "SymbolTypes" = DEFAULT_SCOPE,
    to_symbol_types: "OptSymbolTypes" = SYMBOL_TYPE_VALUES,
    preferred_country_code: "OptCountryCode" = None,
    asset_class: "OptAssetClass" = None,
    asset_state: "OptAssetState" = None,
):
    """
    This function describes parameters to convert symbols

    Parameters
    ----------
    symbols: str or list of str
        Single instrument or list of instruments to convert.

    from_symbol_type: SymbolTypes, optional
        Instrument code to convert from.
        Possible values: 'CUSIP', 'ISIN', 'SEDOL', 'RIC', 'ticker', 'lipperID', 'IMO'
        Default: '_AllUnique'

    to_symbol_types: SymbolTypes, str or list of str or SymbolTypes, optional
        Instrument code to convert to.
        Possible values: 'CUSIP', 'ISIN', 'SEDOL', 'RIC', 'ticker', 'lipperID', 'IMO', 'OAPermID'
        Default: all symbol types are requested

    preferred_country_code: CountryCode, optional
        Unique ISO 3166 code for country

    asset_class: AssetClass, optional
        AssetClass value to build filter parameter.

    asset_state: AssetState, optional
        AssetState value to build filter parameter.

    Returns
    -------
    pd.DataFrame

    Examples
    --------
    >>> import refinitiv.data as rd
    >>> df = rd.discovery.convert_symbols(symbols=["US5949181045", "US02079K1079"], to_symbol_types="RIC")

    """
    return (
        content.symbol_conversion.Definition(
            symbols=symbols,
            from_symbol_type=from_symbol_type,
            to_symbol_types=to_symbol_types,
            preferred_country_code=preferred_country_code,
            asset_class=asset_class,
            asset_state=asset_state,
        )
        .get_data()
        .data.df
    )



from refinitiv.data.content.search import metadata


def test_workspace():
    assert hasattr(metadata, "Definition")


def test_attributes():
    # given
    excepted_attributes = [
        "_view",
        #
        "_kwargs",
        "_data_type",
        "_content_type",
        "_provider",
    ]

    # when
    definition = metadata.Definition(view=...)
    attributes = list(definition.__dict__.keys())

    # then
    assert len(attributes) == len(excepted_attributes)
    assert set(attributes) == set(excepted_attributes)


def test_definition_metadata_repr():
    # given
    definition = metadata.Definition(view=...)
    obj_id = hex(id(definition))
    expected_value = f"<refinitiv.data.content.search.metadata.Definition object at {obj_id} {{view='Ellipsis'}}>"

    # when
    testing_value = repr(definition)

    # then
    assert testing_value == expected_value



import allure
import pytest

from refinitiv.data.content.ownership.consolidated import shareholders_report
from tests.integration.constants_list import HttpStatusCode, HttpReason
from tests.integration.content.ownership.helpers import (
    check_response_dataframe_instrument_contains_expected_universes,
    get_expected_column,
)
from tests.integration.helpers import (
    get_async_response_from_definition,
    check_response_status,
    check_non_empty_response_data,
    check_response_dataframe_contains_columns_names,
)

EXPECTED_COLUMN_NAMES = [
    "instrument",
    "investorid",
    "TR.InvestorFullName",
    "TR.InvParentType",
    "TR.InvestorType",
    "TR.InvInvestmentStyleCode",
    "TR.InvInvmtOrientation",
    "TR.OwnTrnverRating",
    "TR.OwnTurnover",
    "TR.InvestorRegion",
    "TR.TotalEquityAssets",
    "TR.SharesHeld",
    "TR.PctOfSharesOutHeld",
    "TR.ConsHoldPctPortfolio",
    "TR.SharesHeldValue",
    "TR.HoldingsDate",
    "TR.FilingType",
    "TR.SharesHeldChange",
    "TR.SharesHeldValChg",
    "TR.PctSharesOutHeldChange",
    "TR.WeightCountry",
    "TR.PctCountry",
    "TR.WeightSector",
    "TR.PctSector",
    "TR.WeightExchange",
    "TR.PctExchange",
    "TR.PrevSharesHeld",
    "TR.PrevPctOfSharesOutHeld",
    "TR.PrevSharesHeldValue",
    "TR.PrevHoldingsDate",
    "TR.PrevFilingType",
    "TR.InvContFirstName",
    "TR.InvContMidInit",
    "TR.InvContLastName",
    "TR.InvContTelCntry",
    "TR.InvContAreaCode",
    "TR.InvContTelNumber",
    "TR.InvContTelExt",
    "TR.InvConAddrMetroArea",
    "TR.InvestorAddrCity",
    "TR.InvAddrCountry",
]

EXPECTED_COLUMN_TITLES = [
    "Instrument",
    "Calc Date",
    "Investor Id",
    "Investor Perm Id",
    "Investor Name",
    "Investor Parent Type",
    "Investor Type",
    "Investment Style",
    "Orientation",
    "Turnover Rating",
    "Turnover %",
    "Investor Region",
    "Total Equity Assets",
    "SharesHeld",
    "% of SharesOutstanding",
    "Holdings % Portfolio",
    "SharesHeld Value",
    "Holdings Date",
    "Filing Type",
    "SharesHeld Change",
    "SharesHeld Value Change",
    "% SharesOutstanding Change",
    "Weight Country",
    "% Country",
    "Weight Sector",
    "% Sector",
    "Weight Exchange",
    "% Exchange",
    "Previous SharesHeld",
    "Previous % SharesOutstanding",
    "Previous SharesHeld Value",
    "Previous Holdings Date",
    "Previous Filing Type",
    "Contact First Name",
    "Contact Middle Name",
    "Contact Last Name",
    "Contact Phone Country",
    "Contact Phone AreaCode",
    "Contact Phone TelNumber",
    "Contact PhoneExt",
    "Contact MetroArea",
    "Investor City",
    "Investor Country",
    "UltimateParentId",
]


@allure.suite("Content object - Ownership Consolidated Shareholders Report")
@allure.feature("Content object - Ownership Consolidated Shareholders Report")
@allure.severity(allure.severity_level.CRITICAL)
class TestConsolidatedShareholdersReport:
    @allure.title(
        "Create Consolidated Shareholders Report definition object with valid params and get data"
    )
    @pytest.mark.caseid("37556963")
    @pytest.mark.parametrize(
        "universe,limit,use_field_names_in_headers,extended_params,expected_universe",
        [
            ("ONEX.CCP", 5, True, {"universe": "VOD.L"}, "VOD.L"),
            (["IBM.N", "TRI.N"], 100, True, {"offset": 3560}, ["IBM.N", "TRI.N"]),
            (["ONEX.CCP", "invalid universe"], 100, False, {"offset": 50}, "ONEX.CCP"),
        ],
    )
    def test_create_consolidated_shareholders_report_object_with_valid_fields_and_get_data(
        self,
        open_platform_session,
        universe,
        limit,
        use_field_names_in_headers,
        extended_params,
        expected_universe,
    ):
        expected_column = get_expected_column(
            use_field_names_in_headers, EXPECTED_COLUMN_TITLES, EXPECTED_COLUMN_NAMES
        )
        response = shareholders_report.Definition(
            universe, limit, use_field_names_in_headers, extended_params
        ).get_data()
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_response_dataframe_contains_columns_names(response, expected_column)
        check_response_dataframe_instrument_contains_expected_universes(
            response, expected_universe
        )

    @allure.title(
        "Create Consolidated Shareholders Report definition object with valid params and get data asynchronously"
    )
    @pytest.mark.caseid("37556964")
    @pytest.mark.parametrize(
        "universe",
        [
            ("ONEX.CCP"),
            pytest.param(
                ["TRI.N", "ONEX.CCP", "invalid"],
                marks=pytest.mark.skip(
                    reason="https://jira.refinitiv.com/browse/ADC-54133"
                ),
            ),
        ],
    )
    # this test fails due to server-side issue https://jira.refinitiv.com/browse/ADC-54133,
    # uncomment test params when issue is fixed.
    async def test_create_consolidated_shareholders_report_object_with_valid_fields_and_get_data_async(
        self, open_platform_session_async, universe
    ):
        response = await get_async_response_from_definition(
            shareholders_report.Definition(universe, use_field_names_in_headers=True)
        )
        check_response_status(response, HttpStatusCode.TWO_HUNDRED, HttpReason.OK)
        check_non_empty_response_data(response)
        check_response_dataframe_contains_columns_names(response, EXPECTED_COLUMN_NAMES)
        check_response_dataframe_instrument_contains_expected_universes(
            response, universe
        )



import pytest

from refinitiv.data.content._df_builder import build_empty_df
from refinitiv.data.content.news._news_data_provider import (
    NewsHeadlinesRDPData,
    NewsStoryResponseFactory,
    NewsHeadlinesUDFRequestFactory,
    NewsStoryUDFRequestFactory,
    NewsStoryRDPRequestFactory,
    NewsHeadlinesRDPRequestFactory,
    NewsUDFContentValidator,
    NewsStoryResponse,
    NewsStoryRDPData,
    NewsUDFDataProvider,
    NewsHeadlinesRDPDataProvider,
)
from refinitiv.data.delivery._data._data_factory import DataFactory
from refinitiv.data.delivery._data._data_provider import Response, ParsedData
from tests.unit.conftest import StubSession
from ...news.test_news import STORY_RAW_JSON, HEADLINE_RAW_JSON

ERROR_MESSAGE_FOR_NEWS_STORY = "Error while calling the NEP backend: Story not found"


def test_news_story_rdp_data():
    # given
    # when
    news_story_data = NewsStoryRDPData(STORY_RAW_JSON)

    # then
    assert hasattr(news_story_data, "raw")
    assert hasattr(news_story_data, "story")


def test_news_headlines_rdp_data():
    # given
    # when
    news_headlines_data = NewsHeadlinesRDPData(HEADLINE_RAW_JSON, build_empty_df)

    # then
    assert hasattr(news_headlines_data, "raw")
    assert hasattr(news_headlines_data, "headlines")


@pytest.mark.parametrize(
    ("input_error_code", "input_error_message"),
    [
        (400, ""),
        (400, "a"),
        (400, ERROR_MESSAGE_FOR_NEWS_STORY),
        (404, ""),
        (404, "a"),
        (404, ERROR_MESSAGE_FOR_NEWS_STORY),
    ],
)
def test_news_story_response_factory_create_fail_changed_data(
    input_error_code, input_error_message
):
    # given
    data = ParsedData(
        {},
        {},
        **{
            "error_codes": input_error_code,
            "error_messages": input_error_message,
        },
    )

    # when
    response_factory = NewsStoryResponseFactory()
    response_factory.create_fail(data)

    # then
    assert data.first_error_code == 404
    assert data.first_error_message == ERROR_MESSAGE_FOR_NEWS_STORY


def test_news_story_response__str_fail_data():
    # given
    input_data = ParsedData(
        {},
        {},
        **{
            "error_codes": 404,
            "error_messages": ERROR_MESSAGE_FOR_NEWS_STORY,
        },
    )
    expected_str = f"[Error(code=404, message='{ERROR_MESSAGE_FOR_NEWS_STORY}')]"

    # when
    new_story_response = NewsStoryResponse(False, input_data, DataFactory())

    # then
    assert str(new_story_response) == expected_str


@pytest.mark.parametrize(
    ("input_data", "expected_result"),
    [
        (
            {
                "query": "test_query",
                "count": 11,
                "repository": "test_repo",
                "date_from": "11-11-2020",
                "date_to": "11-11-2020",
                "payload": "/headlines?payload=TEST",
            },
            {
                "Entity": {
                    "E": "News_Headlines",
                    "W": {
                        "query": "test_query",
                        "number": "11",
                        "repository": "test_repo",
                        "productName": "app_key",
                        "dateFrom": "2020-11-11T00:00:00",
                        "dateTo": "2020-11-11T00:00:00",
                        "payload": "TEST",
                    },
                }
            },
        )
    ],
)
def test_news_headlines_request_factory_udf_get_body_parameters(
    input_data, expected_result
):
    # given
    session = StubSession()

    # when
    result = NewsHeadlinesUDFRequestFactory().get_body_parameters(session, **input_data)

    # then
    assert result == expected_result


@pytest.mark.parametrize(
    ("input_data", "expected_result"),
    [
        (
            {
                "story_id": "test_story_id",
            },
            {
                "Entity": {
                    "E": "News_Story",
                    "W": {
                        "storyId": "test_story_id",
                        "productName": "app_key",
                    },
                }
            },
        )
    ],
)
def test_news_story_request_factory_udf_get_body_parameters(
    input_data, expected_result
):
    # given
    session = StubSession()

    # when
    result = NewsStoryUDFRequestFactory().get_body_parameters(session, **input_data)

    # then
    assert result == expected_result


@pytest.mark.parametrize(
    ("input_data", "expected_result"),
    [
        (
            {
                "story_id": "test_story_id",
            },
            {"storyId": "test_story_id"},
        )
    ],
)
def test_news_story_request_factory_rdp_get_path_parameters(
    input_data, expected_result
):
    # given
    # when
    result = NewsStoryRDPRequestFactory().get_path_parameters(**input_data)

    # then
    assert result == expected_result


def test_news_story_request_factory_rdp_get_header_parameters():
    # given
    # when
    result = NewsStoryRDPRequestFactory().get_header_parameters()

    # then
    assert result == {"accept": "application/json"}


@pytest.mark.parametrize(
    ("input_data", "expected_result"),
    [
        (
            {
                "query": "test_query",
                "count": 10,
                "date_from": "11-11-2020",
                "date_to": "11-11-2020",
                "sort_order": "oldToNew",
                "cursor": "test",
            },
            [
                ("query", "test_query"),
                ("limit", 10),
                ("dateFrom", "2020-11-11T00:00:00"),
                ("dateTo", "2020-11-11T00:00:00"),
                ("sort", "oldToNew"),
                ("cursor", "test"),
            ],
        )
    ],
)
def test_news_headlines_request_factory_rdp_get_query_parameters(
    input_data, expected_result
):
    # given
    session = StubSession()

    # when
    result = NewsHeadlinesRDPRequestFactory().get_query_parameters(
        session, **input_data
    )

    # then
    assert result == expected_result


def test_news_headlines_request_factory_udf_extend_body_parameters():
    # given
    body_parameters = {"Entity": {"W": {"key": "value"}}}
    extended_params = {"new_key": "new_value"}

    expected_result = {"Entity": {"W": {"key": "value", "new_key": "new_value"}}}

    # when
    result = NewsHeadlinesUDFRequestFactory().extend_body_parameters(
        body_parameters, extended_params
    )

    # then
    assert result == expected_result


def test_news_headlines_request_factory_rdp_extend_query_parameters():
    # given
    query_parameters = [("key", "value"), ("number", 5)]
    extended_params = {"number": 20}

    expected_result = [("key", "value"), ("number", 20)]

    # when
    result = NewsHeadlinesRDPRequestFactory().extend_query_parameters(
        query_parameters, extended_params
    )

    # then
    assert result == expected_result


def test_news_headlines_request_factory_rdp_extend_body_parameters():
    # given
    expected_parameters = {"number": 5}
    extended_params = {"number": 20}

    # when
    result = NewsHeadlinesRDPRequestFactory().extend_body_parameters(
        expected_parameters, extended_params=extended_params
    )

    # then
    assert result == expected_parameters


@pytest.mark.parametrize(
    ("input_data", "expected_result"),
    [
        (
            {
                "content_data": {
                    "ErrorCode": 404,
                    "ErrorMessage": "Error code 404 | Backend error. 404 Not Found",
                }
            },
            False,
        ),
        ({"content_data": None}, False),
        ({"content_data": {"test": "data"}}, True),
    ],
)
def test_news_content_validator_udf(input_data, expected_result):
    # given
    parsed_data = ParsedData({}, {}, **input_data)
    # when
    result = NewsUDFContentValidator().validate(parsed_data)

    # then
    assert result == expected_result


@pytest.mark.parametrize(
    "input_count, limit, expected_count",
    [
        (1, 2, 1),
        (5, 10, 5),
        (15, 10, 5),
        (1, 100, 99),
        (1, 101, 0),
        (101, 100, 1),
        (100, 1000, 0),
    ],
)
def test_news_data_provider_udf_change_count(input_count, limit, expected_count):
    # given
    kwargs = {"count": 0}

    # when
    NewsUDFDataProvider().change_count(count=input_count, limit=limit, kwargs=kwargs)

    # then
    assert kwargs["count"] == expected_count


def test_news_data_provider_udf_get_data_without_count():
    # given
    session = StubSession(is_open=True)

    # when
    result = NewsUDFDataProvider().get_data(session, "")

    # then
    assert isinstance(result, Response)


def test_news_data_provider_udf_get_data_with_count_zero():
    # given
    session = StubSession(is_open=True)

    # when
    result = NewsUDFDataProvider().get_data(session, "", **{"count": 0})

    # then
    assert isinstance(result, Response)
    assert result.is_success is True
    assert result.data._dataframe is None
    assert result.errors == []


def test_news_data_provider_rdp_get_data_with_count_zero():
    # given
    session = StubSession(is_open=True)

    # when
    result = NewsHeadlinesRDPDataProvider().get_data(session, "", **{"count": 0})

    # then
    assert isinstance(result, Response)
    assert result.is_success is True
    assert result.data._dataframe is None
    assert result.errors == []



class HttpStatusCode(object):
    TWO_HUNDRED = 200
    FOUR_HUNDRED = 400
    FOUR_HUNDRED_THREE = 403
    FOUR_HUNDRED_FOUR = 404
    FOUR_HUNDRED_TWELVE = 412


class HttpReason(object):
    OK = "OK"
    BAD_REQUEST = "Bad Request"
    NOT_FOUND = "Not Found"


class Intervals(object):
    ONE_MINUTE = "PT1M"
    FIVE_MINUTES = "PT5M"
    TEN_MINUTES = "PT10M"
    THIRTY_MINUTES = "PT30M"
    SIXTY_MINUTES = "PT60M"
    ONE_HOUR = "PT1H"
    DAILY = "P1D"
    SEVEN_DAYS = "P7D"
    WEEKLY = "P1W"
    MONTHLY = "P1M"
    QUARTERLY = "P3M"
    TWELVE_MONTHS = "P12M"
    YEARLY = "P1Y"


ISO_DATE_FORMAT = "%Y-%m-%dT%H:%M:%SZ"



__all__ = (
    "AdjustInterestToPaymentDate",
    "AmortizationFrequency",
    "AmortizationItem",
    "AmortizationType",
    "BenchmarkYieldSelectionMode",
    "BondRoundingParameters",
    "BusinessDayConvention",
    "CreditSpreadType",
    "DateRollingConvention",
    "DayCountBasis",
    "Definition",
    "Direction",
    "DividendType",
    "Frequency",
    "IndexAverageMethod",
    "IndexCompoundingMethod",
    "InflationMode",
    "InterestType",
    "PriceSide",
    "PricingParameters",
    "ProjectedIndexCalculationMethod",
    "QuoteFallbackLogic",
    "RedemptionDateType",
    "Rounding",
    "RoundingType",
    "StubRule",
    "VolatilityTermStructureType",
    "VolatilityType",
    "YieldType",
)

from ._bond_pricing_parameters import PricingParameters
from ._definition import Definition
from ._enums import (
    AmortizationFrequency,
    AmortizationType,
    BenchmarkYieldSelectionMode,
    BusinessDayConvention,
    CreditSpreadType,
    DateRollingConvention,
    DayCountBasis,
    DividendType,
    Frequency,
    IndexAverageMethod,
    InflationMode,
    PriceSide,
    ProjectedIndexCalculationMethod,
    QuoteFallbackLogic,
    RedemptionDateType,
    Rounding,
    RoundingType,
    VolatilityTermStructureType,
    VolatilityType,
    YieldType,
)
from ._models import BondRoundingParameters
from ..._enums import (
    AdjustInterestToPaymentDate,
    Direction,
    IndexCompoundingMethod,
    InterestType,
    StubRule,
)
from ..._models import AmortizationItem



from typing import TYPE_CHECKING

from ...._curves._cross_currency_curves._curves._request import RequestItem
from ......_content_type import ContentType
from ......_tools import create_repr
from ....._content_provider_layer import ContentProviderLayer


if TYPE_CHECKING:
    from ......_types import OptStr, ExtendedParams
    from ...._curves._cross_currency_curves._curves._types import (
        CurveDefinition,
        CurveParameters,
        ShiftScenarios,
    )


class Definition(ContentProviderLayer):
    """
    Generates the Cross Currency curves for the definitions provided

    Parameters
    ----------
    curve_definition : FxForwardCurveDefinition, optional
        FxForwardCurveDefinition object.
    curve_parameters : FxForwardCurveParameters, optional
        FxForwardCurveParameters object.
    shift_scenarios : FxShiftScenario, optional
        The list of attributes applied to the curve shift scenarios.
    curve_tag : str, optional
        A user-defined string to identify the interest rate curve. it can be used to
        link output results to the curve definition. limited to 40 characters. only
        alphabetic, numeric and '- _.#=@' characters are supported.
    extended_params : dict, optional
        If necessary other parameters.

    Examples
    --------
    >>> import refinitiv.data.content.ipa.curves._cross_currency_curves as crs_currency
    >>> definition = crs_currency.curves.Definition(
    ...    curve_definition=crs_currency.curves.FxForwardCurveDefinition(
    ...        base_currency="EUR",
    ...        base_index_name="ESTR",
    ...        quoted_currency="USD",
    ...        quoted_index_name="SOFR"
    ...    ),
    ...    curve_parameters=crs_currency.curves.FxForwardCurveParameters(
    ...        valuation_date="2021-10-06"
    ...    )
    ... )
     >>> response = definition.get_data()

     Using get_data_async
     >>> import asyncio
     >>> task = definition.get_data_async()
     >>> response = asyncio.run(task)
    """

    def __init__(
        self,
        curve_definition: "CurveDefinition" = None,
        curve_parameters: "CurveParameters" = None,
        shift_scenarios: "ShiftScenarios" = None,
        curve_tag: "OptStr" = None,
        extended_params: "ExtendedParams" = None,
    ) -> None:

        request_item = RequestItem(
            curve_definition=curve_definition,
            curve_parameters=curve_parameters,
            shift_scenarios=shift_scenarios,
            curve_tag=curve_tag,
        )
        super().__init__(
            content_type=ContentType.CROSS_CURRENCY_CURVES_CURVES,
            universe=request_item,
            extended_params=extended_params,
        )

    def __repr__(self):
        return create_repr(self, middle_path="_cross_currency_curves.curves")



import json
import time
from typing import Callable, TYPE_CHECKING

import requests

from .event import UpdateEvent
from .tools import NullResponse, UNAUTHORIZED_CODES
from .updater import Updater
from ..log_reporter import LogReporter
from ...delivery._data._endpoint_data import RequestMethod
from ...delivery._data._request import Request

if TYPE_CHECKING:
    from .auth_manager import TokenInfo
    from ._platform_session import PlatformSession
    from .grant_password import GrantPassword

codes = requests.codes


class RefreshTokenUpdater(Updater, LogReporter):
    def __init__(
        self,
        session: "PlatformSession",
        token_info: "TokenInfo",
        delay: float,
        callback: Callable[[str, str, dict], None],
    ) -> None:
        Updater.__init__(self, delay, "RefreshTokenUpdater")
        LogReporter.__init__(self, logger=session)

        self._session = session
        self._token_info = token_info
        self._callback = callback
        self._grant: "GrantPassword" = self._session._grant
        self._app_key = self._session._app_key
        self._url = self._session.authentication_token_endpoint_url

    @Updater.delay.setter
    def delay(self, value: int):
        if value <= 0:
            raise ValueError("Delay must be greater than 0")
        Updater.delay.fset(self, value)

    def _do_update(self):
        cur_time = time.time()

        if self._token_info.expires_at <= cur_time:
            event = UpdateEvent.REFRESH_TOKEN_EXPIRED
            message = "Time expired for the refresh token update"
            self.debug(message)
            self._callback(event, message, {})
            return

        response = self._request_token(
            self._grant, self._token_info, self._app_key, self._url
        )

        try:
            json_content = response.json()
        except json.decoder.JSONDecodeError:
            message = (
                f"Malformed JSON received during token refresh: '{response.text}'. "
                f"Status code: {response.status_code}"
            )
            self.error(message)
            self._callback(UpdateEvent.REFRESH_TOKEN_FAILED, message, {})
            return

        status_code = response.status_code

        if status_code == codes.ok:
            event = UpdateEvent.REFRESH_TOKEN_SUCCESS
            message = "All is well"

        elif status_code in UNAUTHORIZED_CODES:
            event = UpdateEvent.REFRESH_TOKEN_BAD
            error = json_content.get("error")
            error_description = json_content.get(
                "error_description", "empty error description"
            )
            message = error_description
            self.error(f"[Error {status_code} - {error}] {error_description}")

        else:
            event = UpdateEvent.REFRESH_TOKEN_FAILED
            error = json_content.get("error")
            error_description = json_content.get(
                "error_description",
                getattr(response, "text", "empty error description"),
            )
            message = error_description
            self.error(f"[Error {status_code} - {error}] {error_description}")

        self._callback(event, message, json_content)

    def _request_token(
        self,
        grant: "GrantPassword",
        token_info: "TokenInfo",
        app_key: str,
        url: str,
    ):
        data = {
            "client_id": app_key,
            "grant_type": "refresh_token",
            "username": grant.get_username(),
            "refresh_token": token_info.refresh_token,
        }
        self.debug(f"Request refresh token to {url}\n\twith post data = {str(data)}")
        headers = {
            "Accept": "application/json",
            "Authorization": f"Bearer {token_info.access_token}",
        }
        try:
            request = Request(
                url=url,
                method=RequestMethod.POST,
                headers=headers,
                data=data,
                auto_retry=True,
            )
            response = self._session.http_request(request)
            self.debug(f"Refresh token response: {response.text}")
        except Exception as e:
            response = NullResponse()
            response.text = str(e)

        return response

    def _do_dispose(self):
        self._session = None
        self._callback = None



# coding: utf8

from enum import Enum, unique


@unique
class DocClause(Enum):
    CUM_RESTRUCT03 = "CumRestruct03"
    CUM_RESTRUCT14 = "CumRestruct14"
    EX_RESTRUCT03 = "ExRestruct03"
    EX_RESTRUCT14 = "ExRestruct14"
    MOD_MOD_RESTRUCT03 = "ModModRestruct03"
    MOD_MOD_RESTRUCT14 = "ModModRestruct14"
    MODIFIED_RESTRUCT03 = "ModifiedRestruct03"
    MODIFIED_RESTRUCT14 = "ModifiedRestruct14"
    NONE = "None"



import inspect
from unittest import mock

import pytest

from refinitiv.data.content.pricing import chain
from refinitiv.data.content.pricing.chain._chain_record import (
    _create_chain_record,
    config_17_chars,
)
from refinitiv.data.content.pricing.chain._stream_facade import Stream
from tests.unit.conftest import StubSession


def test_chain_record_num_constituents_is_not_none():
    fields = {
        "PROD_PERM": 5926,
        "RDNDISPLAY": 173,
        "DSPLY_NAME": "GERMAN VOLUME",
        "RDN_EXCHID": "GER",
        "TIMACT": "13:56:59",
        "CURRENCY": "EUR",
        "ACTIV_DATE": "2022-12-16",
        "NUM_MOVES": 1065349,
        "OFFCL_CODE": "0",
        "REF_COUNT": None,
        "RECORDTYPE": 117,
        "LONGLINK1": None,
        "LONGLINK2": None,
        "LONGLINK3": None,
        "LONGLINK4": None,
        "LONGLINK5": None,
        "LONGLINK6": None,
        "LONGLINK7": None,
        "LONGLINK8": None,
        "LONGLINK9": None,
        "LONGLINK10": None,
        "LONGLINK11": None,
        "LONGLINK12": None,
        "LONGLINK13": None,
        "LONGLINK14": None,
        "LONGPREVLR": "3.AV.DE",
        "PREF_DISP": 173,
        "TIMACT1": "13:56:59",
        "DDS_DSO_ID": 12388,
        "SPS_SP_RIC": ".[SPSXETRAVAE1",
    }
    chain_record = _create_chain_record(fields, config_17_chars)
    assert chain_record.num_constituents is not None
    assert chain_record.num_constituents == 0


def test_chain_stream_has_constituents_property():
    # given
    session = StubSession(is_open=True)
    stream = Stream("name", session=session)
    stream._stream.get_constituents = lambda: []

    try:
        stream.constituents
    except AttributeError as e:
        assert False, str(e)

    else:
        assert True


def test_delete_streaming_chain_class_from_public_layer():
    # given
    import refinitiv.data as rd

    # then
    with pytest.raises(AttributeError):
        # when
        rd.content.pricing.chain.StreamingChain


def test_attributes_definition():
    # given
    expected_attributes = [
        "_name",
        "_service",
        "_skip_summary_links",
        "_skip_empty",
        "_override_summary_links",
        "_extended_params",
    ]

    # when
    definition = chain.Definition("")
    attributes = list(definition.__dict__.keys())

    # then
    assert attributes == expected_attributes


def test_inspect_init_chain():
    # given
    expected_attributes = [
        "name",
        "service",
        "skip_summary_links",
        "skip_empty",
        "override_summary_links",
        "extended_params",
    ]
    inspect_pricing_init = inspect.signature(chain.Definition.__init__)

    # when
    attributes = inspect_pricing_init.parameters

    # then
    assert all(x in attributes for x in expected_attributes)


def test_smoke_get_stream_into_chain():
    # given
    session = StubSession(is_open=True)
    definition = chain.Definition("name")

    with mock.patch(
        "refinitiv.data.content.pricing.chain._stream_facade.Stream.__init__",
        return_value=None,
    ) as mock_stream:
        # when
        definition.get_stream(session=session)

        # then
        mock_stream.assert_called_once_with(
            name="name",
            session=session,
            service=None,
            skip_summary_links=True,
            skip_empty=True,
            override_summary_links=None,
            extended_params=None,
        )


def test_chain_repr():
    # given
    definition_chain = chain.Definition("0#.FTSE")
    obj_id = hex(id(definition_chain))
    expected_repr = (
        f"<refinitiv.data.content.pricing.chain.Definition object at "
        f"{obj_id} {{name='0#.FTSE'}}>"
    )

    # when
    s = repr(definition_chain)

    # then
    assert s == expected_repr



import logging
import os
import sys
from datetime import datetime
from functools import lru_cache
from logging import LogRecord
from logging.handlers import RotatingFileHandler, TimedRotatingFileHandler
from typing import Tuple, Any, Optional

from ._tools import DEBUG
from ._tools._specification import BaseSpecification

# ---------------------------------------------------------------------------
#   Conversion from TS/JS to Python
# ---------------------------------------------------------------------------

TRACE = 5

py_grade = logging._nameToLevel.copy()
py_grade["TRACE"] = TRACE
# add an additional level
logging.addLevelName(TRACE, "TRACE")
py_grade = {
    f"py_{lname}": {"name": lname, "level": llevel}
    for lname, llevel in py_grade.items()
}

ts_grade = {
    "ts_trace": {"name": "trace", "level": 0},
    "ts_debug": {"name": "debug", "level": 1},
    "ts_info": {"name": "info", "level": 2},
    "ts_warn": {"name": "warn", "level": 3},
    "ts_error": {"name": "error", "level": 4},
    "ts_silent": {"name": "silent", "level": 5},
}

conversion_schema = [
    ("ts_trace", "py_TRACE"),
    ("ts_debug", "py_DEBUG"),
    ("ts_info", "py_INFO"),
    ("ts_warn", "py_WARNING"),
    ("ts_error", "py_ERROR"),
    ("ts_silent", "py_CRITICAL"),
]

"""
+------------+----------+
| TypeScript | Python   |
+------------+----------+
| trace      | TRACE    |
+------------+----------+
| debug      | DEBUG    |
+------------+----------+
| info       | INFO     |
+------------+----------+
| warn       | WARNING  |
+------------+----------+
| error      | ERROR    |
+------------+----------+
| silent     | CRITICAL |
+------------+----------+
"""
py_by_ts_nameToName = {
    ts_grade[ts_]["name"]: py_grade[py_]["name"] for ts_, py_ in conversion_schema
}

"""
+------------+--------+
| TypeScript | Python |
+------------+--------+
| 0          | 5      |
+------------+--------+
| 1          | 10     |
+------------+--------+
| 2          | 20     |
+------------+--------+
| 3          | 30     |
+------------+--------+
| 4          | 40     |
+------------+--------+
| 5          | 50     |
+------------+--------+
"""
py_by_ts_levelToLevel = {
    ts_grade[ts_]["level"]: py_grade[py_]["level"] for ts_, py_ in conversion_schema
}

# ---------------------------------------------------------------------------
#   File handler
# ---------------------------------------------------------------------------


bytes_by_suffix = {
    "B": 1,  # B
    "K": 2**10,  # KiB
    "M": 2**20,  # MiB
    "G": 2**30,  # GiB
}


def convert_filesize(s) -> int:
    if isinstance(s, int):
        return s

    if isinstance(s, str):
        suffix_ = s[-1]
        count_ = int(s[:-1])
        bytes_ = bytes_by_suffix[suffix_]
        count_bytes_ = count_ * bytes_
        return count_bytes_


def convert_interval(s) -> Tuple[int, str]:
    when_ = s[-1]
    interval_ = int(s[:-1])

    # Months
    if when_ == "M":
        when_ = "D"
        interval_ = interval_ * 30

    return interval_, when_


class TimedSizedRotatingHandler(TimedRotatingFileHandler, RotatingFileHandler):
    def __init__(
        self,
        filename,
        file_mode="a",
        max_bytes=0,
        backup_count=0,
        encoding="ascii",
        delay=False,
        when="h",
        interval=1,
        utc=False,
        at_time=None,
        *args,
        **kwargs,
    ):

        if file_mode.startswith("w"):
            try:
                os.remove(filename)
            except Exception:
                pass

        self.filename = filename
        RotatingFileHandler.__init__(
            self,
            filename=filename,
            mode=file_mode,
            maxBytes=max_bytes,
            backupCount=backup_count,
            encoding=encoding,
            delay=delay,
        )
        TimedRotatingFileHandler.__init__(
            self,
            filename=filename,
            when=when,
            interval=interval,
            backupCount=backup_count,
            encoding=encoding,
            delay=delay,
            utc=utc,
            atTime=at_time,
        )

        if os.path.sep in filename:
            self.baseFilename = filename

    def shouldRollover(self, record):
        timed_rollover = TimedRotatingFileHandler.shouldRollover(self, record)
        sized_rollover = RotatingFileHandler.shouldRollover(self, record)
        return timed_rollover or sized_rollover

    def doRollover(self):
        super(TimedRotatingFileHandler, self).doRollover()

    def getFilesToDelete(self):
        return super(TimedRotatingFileHandler, self).getFilesToDelete()


def _filenamer(base_filename):
    basename_ = os.path.basename(base_filename)
    date_, time_, pid_, *name_, name_with_count_ = basename_.split("-")
    *name_chunk_, count_ = name_with_count_.split(".")
    name_.append(".".join(name_chunk_))
    name_ = "-".join(name_)
    new_basename_ = "-".join([date_, time_, count_, pid_, name_])
    return base_filename.replace(basename_, new_basename_)


if DEBUG:
    fmt = (
        "[%(asctime)s|"
        "%(levelname)s|"
        "%(thread)d-%(threadName)s|"
        "%(name)s] "
        "%(module)s."
        "%(funcName)s "
        "%(message)s"
    )
else:
    fmt = (
        "[%(asctime)s] - "
        "[%(name)s] - "
        "[%(levelname)s] - "
        "[%(thread)d - %(threadName)s] - "
        "[%(module)s] - "
        "[%(funcName)s] - "
        "%(message)s"
    )


class RDFormatter(logging.Formatter):
    def formatTime(self, record: LogRecord, datefmt: Optional[str] = None) -> str:
        return datetime.fromtimestamp(record.created).astimezone().isoformat()


_file_handler_formatter = RDFormatter(fmt)


def _get_filename(filename_: str, datetime_: datetime, pid_: int) -> str:
    date_ = datetime_.strftime("%Y%m%d")
    time_ = datetime_.strftime("%H%M")
    filename_ = filename_.replace("\\", os.path.sep)
    filename_ = os.path.normpath(filename_)
    *path, filename = filename_.split(os.path.sep)

    if path:
        new_filename = f"{date_}-{time_}-{pid_}-{filename}"
        path.append(new_filename)
        filename_ = f"{os.path.sep}".join(path)

    else:
        filename_ = f"{date_}-{time_}-{pid_}-{filename}"

    return filename_


@lru_cache(None)
def _create_log_file_handler(name_, file_size_, max_files_, interval_):
    # file name
    filename_ = _get_filename(name_, datetime.now(), os.getpid())

    # file size
    file_size_ = convert_filesize(file_size_)

    # interval
    interval_, when_ = convert_interval(interval_)

    handler_ = TimedSizedRotatingHandler(
        filename_,
        max_bytes=file_size_,
        when=when_,
        interval=interval_,
        backup_count=max_files_,
        encoding="utf-8",
        delay=True,
    )
    handler_.namer = _filenamer
    handler_.setFormatter(_file_handler_formatter)
    return handler_


# ---------------------------------------------------------------------------
#   Stdout handler
# ---------------------------------------------------------------------------

if DEBUG:
    fmt = (
        "[%(asctime)s|"
        "%(levelname)s|"
        "%(thread)d-%(threadName)s|"
        "%(name)s] \n"
        "%(module)s."
        "%(funcName)s "
        "%(message)s"
    )
else:
    fmt = (
        "[%(asctime)s] - "
        "[%(levelname)s] - "
        "[%(name)s] - "
        "[%(thread)d] | "
        "%(threadName)s\n"
        "%(message)s"
    )

_stdout_formatter = RDFormatter(fmt)


def _create_log_stdout_handler():
    handler_ = logging.StreamHandler(sys.stdout)
    handler_.setFormatter(_stdout_formatter)
    return handler_


# ---------------------------------------------------------------------------
#   Filtering
# ---------------------------------------------------------------------------


class NotLog(BaseSpecification):
    def is_satisfied_by(self, record: Any) -> bool:
        return False


class LogEverything(BaseSpecification):
    def is_satisfied_by(self, record: Any) -> bool:
        return True


class NotLogWithName(BaseSpecification):
    def __init__(self, name) -> None:
        super().__init__()
        self.name = name

    def is_satisfied_by(self, record: Any) -> bool:
        return self.name != record.name


class LogWithName(BaseSpecification):
    def __init__(self, name) -> None:
        super().__init__()
        self.name = name

    def is_satisfied_by(self, record: Any) -> bool:
        return self.name == record.name


class LogStartsWithName(BaseSpecification):
    def __init__(self, name) -> None:
        super().__init__()
        self.name = name

    def is_satisfied_by(self, record: Any) -> bool:
        return record.name.startswith(self.name)


class NotLogStartsWithName(BaseSpecification):
    def __init__(self, name) -> None:
        super().__init__()
        self.name = name

    def is_satisfied_by(self, record: Any) -> bool:
        return not record.name.startswith(self.name)


default_log_filter = "*"


def join_by_and_(prev_spec, spec):
    return prev_spec and prev_spec.and_(spec) or spec


def join_by_or_(prev_spec, spec):
    return prev_spec and prev_spec.or_(spec) or spec


def make_filter(text):
    ss = [s.strip() for s in text.split(",") if s]

    if not ss:
        can_log = NotLog()

    else:
        can_log = None
        for s in ss:
            if s == "*":
                can_log = join_by_or_(can_log, LogEverything())
            elif s.startswith("-") and s.endswith("*"):
                can_log = join_by_and_(can_log, NotLogStartsWithName(s[1:-1]))
            elif s.startswith("-"):
                can_log = join_by_and_(can_log, NotLogWithName(s[1:]))
            elif s.endswith("*"):
                can_log = join_by_or_(can_log, LogStartsWithName(s[:-1]))
            else:
                can_log = join_by_or_(can_log, LogWithName(s))

    def inner(record):
        return can_log.is_satisfied_by(record)

    return inner


# ---------------------------------------------------------------------------
#   Log level
# ---------------------------------------------------------------------------


def convert_log_level(level) -> int:
    py_level_ = None

    if isinstance(level, str):
        level_ = level.strip()
        py_level_ = py_by_ts_nameToName.get(level_)

        if py_level_ is None:
            py_level_ = level

        py_level_ = logging._nameToLevel.get(py_level_)

    elif isinstance(level, int):
        py_level_ = level

    return py_level_ or logging.INFO


def read_log_level_config():
    from . import _configure as configure

    level_ = configure.get_str(configure.keys.log_level)
    return convert_log_level(level_)


# ---------------------------------------------------------------------------
#   Create and dispose logger
# ---------------------------------------------------------------------------

_log_stream_handler = _create_log_stdout_handler()

_existing_loggers = []


@lru_cache(None)
def create_logger(name):
    from . import _configure as configure

    # construct the logger object for session
    logger_ = logging.getLogger(name)

    log_file_enabled_ = configure.get(configure.keys.log_file_enabled, True)
    if log_file_enabled_:
        name_ = configure.get_str(configure.keys.log_filename)
        file_size_ = configure.get_str(configure.keys.log_file_size)
        max_files_ = configure.get_int(configure.keys.log_max_files)
        interval_ = configure.get_str(configure.keys.log_interval)
        _log_file_handler = _create_log_file_handler(
            name_, file_size_, max_files_, interval_
        )
        logger_.addHandler(_log_file_handler)

    log_console_enabled_ = configure.get(configure.keys.log_console_enabled, True)
    if log_console_enabled_:
        logger_.addHandler(_log_stream_handler)
    else:
        logger_.propagate = False

    log_level_ = read_log_level_config()

    if log_level_ != logger_.level:
        logger_.setLevel(log_level_)

    log_filter_ = configure.get(configure.keys.log_filter, default_log_filter)
    logger_.addFilter(make_filter(log_filter_))

    _existing_loggers.append(name)

    return logger_


def get_logger(name):
    return create_logger(name)


def set_log_level(logger, level):
    if isinstance(logger, str):
        logger = get_logger(logger)

    level = convert_log_level(level)
    logger.setLevel(level)
    return logger


def existing_loggers():
    return _existing_loggers


def dispose_logger(logger):
    if isinstance(logger, str):
        logger = get_logger(logger)

    handlers_ = logger.handlers[:]

    for hdlr_ in handlers_:
        hdlr_.close()
        logger.removeHandler(hdlr_)

    return logger


# ---------------------------------------------------------------------------
#   Root logger
# ---------------------------------------------------------------------------

_root_logger = None


def root_logger():
    global _root_logger

    if _root_logger is None:
        _root_logger = create_logger("rd")

    return _root_logger



from inspect import signature

import pytest

import refinitiv.data.content.ipa.financial_contracts as rdf
from refinitiv.data.delivery._data._data_provider_factory import make_provider
from tests.unit.conftest import StubSession
from tests.unit.conftest import has_property_names_in_class, get_property_names


@pytest.mark.parametrize(
    argnames="input_data",
    ids=[
        "FxBarrierDefinition",
        "FxForwardStart",
        "FxDoubleBarrierInfo",
        "EtiUnderlyingDefinition",
        "FxDoubleBarrierDefinition",
        "FxAverageInfo",
        "BidAskMid",
        "FxUnderlyingDefinition",
        "EtiCbbcDefinition",
        "PricingParameters",
        "EtiDoubleBarriersDefinition",
        "FxDualCurrencyDefinition",
        "EtiBinaryDefinition",
        "EtiFixingInfo",
        "EtiBarrierDefinition",
        "FxBinaryDefinition",
        "FxDoubleBinaryDefinition",
        "InterpolationWeight",
        "PayoutScaling",
        "OptionDefinition",
        "FxDefinition",
        "EtiDefinition",
        "InputFlow",
    ],
    argvalues=[
        (
            rdf.option.FxBarrierDefinition,
            {
                "barrier_mode": rdf.option.BarrierMode.EUROPEAN,
                "in_or_out": rdf.option.InOrOut.OUT,
                "up_or_down": rdf.option.UpOrDown.UP,
                "level": 4.4,
                "rebate_amount": 5.5,
                "window_end_date": "window_end_date",
                "window_start_date": "window_start_date",
            },
        ),
        (
            rdf.option.FxForwardStart,
            {
                "forward_start_date": "forward_start_date",
                "forward_start_tenor": "forward_start_tenor",
                "strike_percent": 123.123,
            },
        ),
        (
            rdf.option.FxDoubleBarrierInfo,
            {
                "in_or_out": rdf.option.InOrOut.IN,
                "level": 2.2,
                "rebate_amount": 3.3,
            },
        ),
        (
            rdf.option.EtiUnderlyingDefinition,
            {
                "instrument_code": "instrument_code",
            },
        ),
        (
            rdf.option.FxDoubleBarrierDefinition,
            {
                "barrier_down": rdf.option.FxDoubleBarrierInfo(),
                "barrier_mode": rdf.option.BarrierMode.EARLY_END_WINDOW,
                "barrier_up": rdf.option.FxDoubleBarrierInfo(),
            },
        ),
        (
            rdf.option.FxAverageInfo,
            {
                "average_type": rdf.option.AverageType.GEOMETRIC_STRIKE,
                "fixing_frequency": rdf.option.FixingFrequency.ANNUAL,
                "average_so_far": 3.3,
                "fixing_ric": "fixing_ric",
                "fixing_start_date": "str",
                "include_holidays": True,
                "include_week_ends": True,
            },
        ),
        (
            rdf.option.BidAskMid,
            {
                "bid": 123.123,
                "ask": 123.123,
                "mid": 123.123,
            },
        ),
        (
            rdf.option.FxUnderlyingDefinition,
            {"fx_cross_code": "fx_cross_code"},
        ),
        (
            rdf.option.EtiCbbcDefinition,
            {
                "conversion_ratio": 1.1,
                "level": 2.2,
            },
        ),
        (
            rdf.option.PricingParameters,
            {
                "butterfly10_d_object": rdf.option.BidAskMid(),
                "butterfly25_d_object": rdf.option.BidAskMid(),
                "payout_custom_dates": ["payout_custom_dates"],
                "payout_scaling_interval": rdf.option.PayoutScaling(),
                "risk_reversal10_d_object": rdf.option.BidAskMid(),
                "risk_reversal25_d_object": rdf.option.BidAskMid(),
                "compute_payout_chart": True,
                "compute_volatility_payout": True,
                "report_ccy_rate": True,
                "atm_volatility_object": rdf.option.BidAskMid(),
                "domestic_deposit_rate_percent_object": rdf.option.BidAskMid(),
                "foreign_deposit_rate_percent_object": rdf.option.BidAskMid(),
                "forward_points_object": rdf.option.BidAskMid(),
                "fx_spot_object": rdf.option.BidAskMid(),
                "fx_swap_calculation_method": rdf.option.FxSwapCalculationMethod.FX_SWAP,
                "implied_volatility_object": rdf.option.BidAskMid(),
                "interpolation_weight": rdf.option.InterpolationWeight(),
                "option_price_side": rdf.option.PriceSide.BID,
                "option_time_stamp": rdf.option.TimeStamp.DEFAULT,
                "price_side": rdf.option.PriceSide.MID,
                "pricing_model_type": rdf.option.PricingModelType.BINOMIAL,
                "underlying_price_side": rdf.option.PriceSide.ASK,
                "underlying_time_stamp": rdf.option.TimeStamp.OPEN,
                "volatility_model": rdf.option.VolatilityModel.CUBIC_SPLINE,
                "volatility_type": rdf.option.OptionVolatilityType.IMPLIED,
                "cutoff_time": "cutoff_time",
                "cutoff_time_zone": "cutoff_time_zone",
                "market_value_in_deal_ccy": 23.23,
                "risk_free_rate_percent": 24.24,
                "underlying_price": 25.25,
                "valuation_date": "valuation_date",
                "volatility_percent": 27.27,
                "market_data_date": "market_data_date",
                "market_value_in_report_ccy": 28.28,
                "volatility": 29.29,
                "report_ccy": "report_ccy",
                "simulate_exercise": True,
            },
        ),
        (
            rdf.option.EtiDoubleBarriersDefinition,
            {
                "barriers_definition": [rdf.option.EtiBarrierDefinition()],
            },
        ),
        (
            rdf.option.FxDualCurrencyDefinition,
            {
                "deposit_start_date": "deposit_start_date",
                "margin_percent": 123.123,
            },
        ),
        (
            rdf.option.EtiBinaryDefinition,
            {
                "notional_amount": 1.1,
                "binary_type": rdf.option.BinaryType.DIGITAL,
                "up_or_down": rdf.option.UpOrDown.DOWN,
                "level": 4.4,
            },
        ),
        (
            rdf.option.EtiFixingInfo,
            {
                "average_so_far": 123,
                "average_type": rdf.option.AverageType.GEOMETRIC_STRIKE,
                "fixing_frequency": rdf.option.FixingFrequency.WEEKLY,
                "fixing_calendar": "fixing_calendar",
                "fixing_end_date": "fixing_end_date",
                "fixing_start_date": "fixing_start_date",
                "include_holidays": True,
                "include_week_ends": True,
            },
        ),
        (
            rdf.option.EtiBarrierDefinition,
            {
                "barrier_style": rdf.option.BarrierStyle.AMERICAN,
                "in_or_out": rdf.option.InOrOut.IN,
                "up_or_down": rdf.option.UpOrDown.DOWN,
                "level": 3.3,
            },
        ),
        (
            rdf.option.FxBinaryDefinition,
            {
                "binary_type": rdf.option.FxBinaryType.ONE_TOUCH_DEFERRED,
                "payout_amount": 3.3,
                "payout_ccy": "payout_ccy",
                "trigger": 5.5,
            },
        ),
        (
            rdf.option.FxDoubleBinaryDefinition,
            {
                "double_binary_type": rdf.option.DoubleBinaryType.DOUBLE_NO_TOUCH,
                "payout_amount": 3.3,
                "payout_ccy": "payout_ccy",
                "trigger_down": 5.5,
                "trigger_up": 6.6,
            },
        ),
        (
            rdf.option.InterpolationWeight,
            {
                "days_list": [rdf.option.DayWeight()],
                "holidays": 123.123,
                "week_days": 123.123,
                "week_ends": 123.123,
            },
        ),
        (
            rdf.option.PayoutScaling,
            {
                "maximum": 123.123,
                "minimum": 123.123,
            },
        ),
        (
            rdf.option._option_definition.OptionDefinition,
            {
                "instrument_tag": "instrument_tag",
                "end_date": "end_date",
                "buy_sell": rdf.option.BuySell.BUY,
                "call_put": rdf.option.CallPut.CALL,
                "exercise_style": rdf.option.ExerciseStyle.AMER,
                "underlying_type": rdf.option.UnderlyingType.ETI,
                "strike": 123.123,
                "start_date": "start_date",
            },
        ),
        (
            rdf.option._fx._fx_definition.FxDefinition,
            {
                "instrument_tag": "instrument_tag",
                "end_date": "end_date",
                "tenor": "tenor",
                "notional_ccy": "notional_ccy",
                "notional_amount": 123.123,
                "asian_definition": rdf.option.FxAverageInfo(),
                "barrier_definition": rdf.option.FxBarrierDefinition(),
                "binary_definition": rdf.option.FxBinaryDefinition(),
                "buy_sell": rdf.option.BuySell.BUY,
                "call_put": rdf.option.CallPut.CALL,
                "double_barrier_definition": rdf.option.FxDoubleBarrierDefinition(),
                "double_binary_definition": rdf.option.FxDoubleBinaryDefinition(),
                "dual_currency_definition": rdf.option.FxDualCurrencyDefinition(),
                "exercise_style": rdf.option.ExerciseStyle.AMER,
                "forward_start_definition": rdf.option.FxForwardStart(),
                "underlying_definition": rdf.option.FxUnderlyingDefinition(),
                "underlying_type": rdf.option.UnderlyingType.ETI,
                "delivery_date": "delivery_date",
                "strike": 123.123,
                "settlement_type": rdf.option.SettlementType.CASH,
                "settlement_ccy": "settlement_ccy",
                "payments": [rdf.option.InputFlow()],
                "start_date": "start_date",
            },
        ),
        (
            rdf.option._eti._eti_definition.EtiDefinition,
            {
                "instrument_tag": "instrument_tag",
                "instrument_code": "instrument_code",
                "end_date": "end_date",
                "asian_definition": rdf.option.EtiFixingInfo(),
                "barrier_definition": rdf.option.EtiBarrierDefinition(),
                "binary_definition": rdf.option.EtiBinaryDefinition(),
                "buy_sell": rdf.option.BuySell.SELL,
                "call_put": rdf.option.CallPut.PUT,
                "cbbc_definition": rdf.option.EtiCbbcDefinition(),
                "double_barriers_definition": rdf.option.EtiDoubleBarriersDefinition(),
                "exercise_style": rdf.option.ExerciseStyle.EURO,
                "underlying_definition": rdf.option.EtiUnderlyingDefinition(),
                "underlying_type": rdf.option.UnderlyingType.FX,
                "deal_contract": 123,
                "end_date_time": "end_date_time",
                "lot_size": 123.123,
                "strike": 123.123,
                "time_zone_offset": 123,
                "start_date": "start_date",
            },
        ),
        (
            rdf.option.InputFlow,
            {
                "amount": 1.1,
                "premium_settlement_type": rdf.option.PremiumSettlementType.SPOT,
                "currency": "currency",
                "date": "date",
            },
        ),
    ],
)
def test_parameter(input_data):
    cls, kwargs = input_data
    args_names = list(kwargs.keys())
    inst = cls(**kwargs)

    s = signature(cls.__init__)
    testing_len = len(s.parameters)
    expected_len = len(args_names) + 1  # +1 for (self)

    if s.parameters.get("kwargs"):
        expected_len += 1  # +1 for kwargs

    assert (
        testing_len == expected_len
    ), f"csl={cls}{str(set(get_property_names(cls)) - set(args_names))}"
    assert has_property_names_in_class(cls, args_names), set(args_names) - set(
        get_property_names(cls)
    )

    for k, v in kwargs.items():
        attr = getattr(inst, k)
        if hasattr(v, "value") and attr == "Eti":
            assert attr == v.value, k
        else:
            assert attr == v, k


def test_request_body():
    # given
    expected_body = {
        "fields": ["1", "2"],
        "universe": [
            {
                "instrumentDefinition": {
                    "asianDefinition": {"averageSoFar": 1.1},
                },
                "extended_params": "extended_params",
                "instrumentType": "Option",
                "pricingParameters": {"marketDataDate": "market_data_date"},
            }
        ],
    }
    definition = rdf.option.Definition(
        asian_definition=rdf.option.EtiFixingInfo(average_so_far=1.1),
        fields=["1", "2"],
        pricing_parameters=rdf.option.PricingParameters(
            market_data_date="market_data_date"
        ),
        extended_params={"extended_params": "extended_params"},
    )

    # when
    content_type = definition._kwargs["__content_type__"]
    provider = make_provider(content_type)
    request = provider.request.create(StubSession(), "url", **definition._kwargs)

    # then
    testing_body = request.json
    assert testing_body == expected_body, expected_body



# coding: utf8

__all__ = ["PeriodType"]

from enum import Enum, unique


@unique
class PeriodType(Enum):
    """
    The method we chose to count the period of time:

    The possible values are:
        - WorkingDay (consider only working days),
        - NonWorkingDay (consider only non working days),
        - Day (consider all days),
        - Year (consider year),
        - NearestTenor (returns the nearest tenor for the given period)
    """

    WORKING_DAY = "WorkingDay"
    NON_WORKING_DAY = "NonWorkingDay"
    DAY = "Day"
    YEAR = "Year"
    NEAREST_TENOR = "NearestTenor"



from enum import Enum

from ...._tools import (
    EnumArgsParser,
    make_parse_enum,
    make_convert_to_enum,
)


class ContribType(Enum):
    REFRESH = "Refresh"
    UPDATE = "Update"


contrib_type_enum_arg_parser = EnumArgsParser(
    parse=make_parse_enum(ContribType),
    parse_to_enum=make_convert_to_enum(ContribType),
)



from ._subcategory import Subcategory
from ._top_news_id import TopNewsId
from ...._content_data import Data
from ...._content_data_provider import ContentDataProvider
from ...._content_response_factory import ContentResponseFactory
from ....._tools import cached_property, ValueParamItem, make_enum_arg_parser
from .....delivery._data._data_provider import RequestFactory


class HierarchyData(Data):
    @cached_property
    def hierarchy(self):
        return {
            category["name"]: {
                page["name"]: Subcategory.from_dict(page) for page in category["pages"]
            }
            for category in self.raw.get("data", [])
        }


top_news_id_arg_parser = make_enum_arg_parser(TopNewsId)

query_params = [ValueParamItem("id", function=top_news_id_arg_parser.get_str)]


class HierarchyRequestFactory(RequestFactory):
    @property
    def query_params_config(self):
        return query_params


news_top_news_hierarchy_data_provider = ContentDataProvider(
    request=HierarchyRequestFactory(),
    response=ContentResponseFactory(data_class=HierarchyData),
)
